# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.costmanagement import CostManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCostManagementCostAllocationRulesOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CostManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_cost_allocation_rules_get(self, resource_group):
        response = self.client.cost_allocation_rules.get(
            billing_account_id="str",
            rule_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_cost_allocation_rules_create_or_update(self, resource_group):
        response = self.client.cost_allocation_rules.create_or_update(
            billing_account_id="str",
            rule_name="str",
            cost_allocation_rule={
                "id": "str",
                "name": "str",
                "properties": {
                    "details": {
                        "sourceResources": [{"name": "str", "resourceType": "str", "values": ["str"]}],
                        "targetResources": [
                            {
                                "name": "str",
                                "policyType": "str",
                                "resourceType": "str",
                                "values": [{"name": "str", "percentage": 0.0}],
                            }
                        ],
                    },
                    "status": "str",
                    "createdDate": "2020-02-20 00:00:00",
                    "description": "str",
                    "updatedDate": "2020-02-20 00:00:00",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_cost_allocation_rules_delete(self, resource_group):
        response = self.client.cost_allocation_rules.delete(
            billing_account_id="str",
            rule_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_cost_allocation_rules_list(self, resource_group):
        response = self.client.cost_allocation_rules.list(
            billing_account_id="str",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_cost_allocation_rules_check_name_availability(self, resource_group):
        response = self.client.cost_allocation_rules.check_name_availability(
            billing_account_id="str",
            cost_allocation_rule_check_name_availability_request={"name": "str", "type": "str"},
        )

        # please add some check logic here by yourself
        # ...
