# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.datashare.aio import DataShareManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDataShareManagementSharesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DataShareManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_shares_list_synchronization_details(self, resource_group):
        response = self.client.shares.list_synchronization_details(
            resource_group_name=resource_group.name,
            account_name="str",
            share_name="str",
            share_synchronization={
                "consumerEmail": "str",
                "consumerName": "str",
                "consumerTenantName": "str",
                "durationMs": 0,
                "endTime": "2020-02-20 00:00:00",
                "message": "str",
                "startTime": "2020-02-20 00:00:00",
                "status": "str",
                "synchronizationId": "str",
                "synchronizationMode": "str",
            },
            api_version="2020-09-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_shares_list_synchronizations(self, resource_group):
        response = self.client.shares.list_synchronizations(
            resource_group_name=resource_group.name,
            account_name="str",
            share_name="str",
            api_version="2020-09-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_shares_get(self, resource_group):
        response = await self.client.shares.get(
            resource_group_name=resource_group.name,
            account_name="str",
            share_name="str",
            api_version="2020-09-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_shares_create(self, resource_group):
        response = await self.client.shares.create(
            resource_group_name=resource_group.name,
            account_name="str",
            share_name="str",
            share={
                "createdAt": "2020-02-20 00:00:00",
                "description": "str",
                "id": "str",
                "name": "str",
                "provisioningState": "str",
                "shareKind": "str",
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "terms": "str",
                "type": "str",
                "userEmail": "str",
                "userName": "str",
            },
            api_version="2020-09-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_shares_begin_delete(self, resource_group):
        response = await (
            await self.client.shares.begin_delete(
                resource_group_name=resource_group.name,
                account_name="str",
                share_name="str",
                api_version="2020-09-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_shares_list_by_account(self, resource_group):
        response = self.client.shares.list_by_account(
            resource_group_name=resource_group.name,
            account_name="str",
            api_version="2020-09-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...
