# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.azurestackhci import AzureStackHCIClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-azurestackhci
# USAGE
    python put_arc_setting.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AzureStackHCIClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.arc_settings.create(
        resource_group_name="test-rg",
        cluster_name="myCluster",
        arc_setting_name="default",
        arc_setting={
            "id": "str",
            "name": "str",
            "properties": {
                "aggregateState": "str",
                "arcApplicationClientId": "str",
                "arcApplicationObjectId": "str",
                "arcApplicationTenantId": "str",
                "arcInstanceResourceGroup": "str",
                "arcServicePrincipalObjectId": "str",
                "connectivityProperties": {
                    "enabled": bool,
                    "serviceConfigurations": [{"port": 0, "serviceName": "str"}],
                },
                "defaultExtensions": [{"category": "str", "consentTime": "2020-02-20 00:00:00"}],
                "perNodeDetails": [
                    {"arcInstance": "str", "arcNodeServicePrincipalObjectId": "str", "name": "str", "state": "str"}
                ],
                "provisioningState": "str",
            },
            "systemData": {
                "createdAt": "2020-02-20 00:00:00",
                "createdBy": "str",
                "createdByType": "str",
                "lastModifiedAt": "2020-02-20 00:00:00",
                "lastModifiedBy": "str",
                "lastModifiedByType": "str",
            },
            "type": "str",
        },
    )
    print(response)


# x-ms-original-file: 2026-04-01-preview/PutArcSetting.json
if __name__ == "__main__":
    main()
