# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import DataProtectionOperationsOperations  # type: ignore
from ._operations import BackupInstancesOperations  # type: ignore
from ._operations import BackupVaultOperationResultsOperations  # type: ignore
from ._operations import DeletedBackupVaultsOperations  # type: ignore
from ._operations import ResourceGuardsOperations  # type: ignore
from ._operations import BackupVaultsOperations  # type: ignore
from ._operations import OperationStatusBackupVaultContextOperations  # type: ignore
from ._operations import ExportJobsOperations  # type: ignore
from ._operations import ExportJobsOperationResultOperations  # type: ignore
from ._operations import BackupPoliciesOperations  # type: ignore
from ._operations import RestorableTimeRangesOperations  # type: ignore
from ._operations import RecoveryPointsOperations  # type: ignore
from ._operations import JobsOperations  # type: ignore
from ._operations import DeletedBackupInstancesOperations  # type: ignore
from ._operations import DppResourceGuardProxyOperations  # type: ignore
from ._operations import OperationResultOperations  # type: ignore
from ._operations import OperationStatusOperations  # type: ignore
from ._operations import OperationStatusResourceGroupContextOperations  # type: ignore
from ._operations import DataProtectionOperations  # type: ignore
from ._operations import FetchSecondaryRecoveryPointsOperations  # type: ignore
from ._operations import FetchCrossRegionRestoreJobOperations  # type: ignore
from ._operations import FetchCrossRegionRestoreJobsOperations  # type: ignore
from ._operations import BackupInstancesExtensionRoutingOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "DataProtectionOperationsOperations",
    "BackupInstancesOperations",
    "BackupVaultOperationResultsOperations",
    "DeletedBackupVaultsOperations",
    "ResourceGuardsOperations",
    "BackupVaultsOperations",
    "OperationStatusBackupVaultContextOperations",
    "ExportJobsOperations",
    "ExportJobsOperationResultOperations",
    "BackupPoliciesOperations",
    "RestorableTimeRangesOperations",
    "RecoveryPointsOperations",
    "JobsOperations",
    "DeletedBackupInstancesOperations",
    "DppResourceGuardProxyOperations",
    "OperationResultOperations",
    "OperationStatusOperations",
    "OperationStatusResourceGroupContextOperations",
    "DataProtectionOperations",
    "FetchSecondaryRecoveryPointsOperations",
    "FetchCrossRegionRestoreJobOperations",
    "FetchCrossRegionRestoreJobsOperations",
    "BackupInstancesExtensionRoutingOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
