# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.costmanagement import CostManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-costmanagement
# USAGE
    python customer_alert_rule_reservation_id_filter.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = CostManagementClient(
        credential=DefaultAzureCredential(),
    )

    response = client.budgets.create_or_update(
        scope="providers/Microsoft.Billing/billingAccounts/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee:ffffffff-gggg-hhhh-iiii-jjjjjjjjjjjj_2023-04-01/customers/000000-1111-2222-3333-444444444444",
        budget_name="TestAlertRule",
        parameters={
            "eTag": '"1d34d016a593709"',
            "properties": {
                "category": "ReservationUtilization",
                "filter": {
                    "dimensions": {
                        "name": "ReservationId",
                        "operator": "In",
                        "values": [
                            "00000000-0000-0000-0000-000000000000",
                            "00000000-0000-0000-0000-000000000001",
                            "00000000-0000-0000-0000-000000000002",
                        ],
                    }
                },
                "notifications": {
                    "Actual_LessThan_99_Percent": {
                        "contactEmails": ["johndoe@contoso.com", "janesmith@contoso.com"],
                        "enabled": True,
                        "frequency": "Daily",
                        "locale": "en-us",
                        "operator": "LessThan",
                        "threshold": 99,
                    }
                },
                "timeGrain": "Last30Days",
                "timePeriod": {"endDate": "2025-04-01T00:00:00Z", "startDate": "2023-04-01T00:00:00Z"},
            },
        },
    )
    print(response)


# x-ms-original-file: 2025-03-01/Budgets/CreateOrUpdate/ReservationUtilization/MCA/Customer-AlertRule-ReservationIdFilter.json
if __name__ == "__main__":
    main()
