# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    Activity,
    ActivityListResult,
    ActivityOutputType,
    ActivityParameter,
    ActivityParameterSet,
    ActivityParameterValidationSet,
    AdvancedSchedule,
    AdvancedScheduleMonthlyOccurrence,
    AgentRegistration,
    AgentRegistrationKeys,
    AgentRegistrationRegenerateKeyParameter,
    AutomationAccount,
    AutomationAccountCreateOrUpdateParameters,
    AutomationAccountListResult,
    AutomationAccountUpdateParameters,
    AzureQueryProperties,
    Certificate,
    CertificateCreateOrUpdateParameters,
    CertificateListResult,
    CertificateUpdateParameters,
    CollectionItemUpdateConfiguration,
    Connection,
    ConnectionCreateOrUpdateParameters,
    ConnectionListResult,
    ConnectionType,
    ConnectionTypeAssociationProperty,
    ConnectionTypeCreateOrUpdateParameters,
    ConnectionTypeListResult,
    ConnectionUpdateParameters,
    ContentHash,
    ContentLink,
    ContentSource,
    Credential,
    CredentialCreateOrUpdateParameters,
    CredentialListResult,
    CredentialUpdateParameters,
    DscCompilationJob,
    DscCompilationJobCreateParameters,
    DscCompilationJobListResult,
    DscConfiguration,
    DscConfigurationAssociationProperty,
    DscConfigurationCreateOrUpdateParameters,
    DscConfigurationListResult,
    DscConfigurationParameter,
    DscConfigurationUpdateParameters,
    DscMetaConfiguration,
    DscNode,
    DscNodeConfiguration,
    DscNodeConfigurationCreateOrUpdateParameters,
    DscNodeConfigurationListResult,
    DscNodeExtensionHandlerAssociationProperty,
    DscNodeListResult,
    DscNodeReport,
    DscNodeReportListResult,
    DscNodeUpdateParameters,
    DscNodeUpdateParametersProperties,
    DscReportError,
    DscReportResource,
    DscReportResourceNavigation,
    ErrorResponse,
    FieldDefinition,
    HybridRunbookWorker,
    HybridRunbookWorkerGroup,
    HybridRunbookWorkerGroupUpdateParameters,
    HybridRunbookWorkerGroupsListResult,
    Job,
    JobCollectionItem,
    JobCreateParameters,
    JobListResultV2,
    JobNavigation,
    JobSchedule,
    JobScheduleCreateParameters,
    JobScheduleListResult,
    JobStream,
    JobStreamListResult,
    Key,
    KeyListResult,
    LinkedWorkspace,
    LinuxProperties,
    Module,
    ModuleCreateOrUpdateParameters,
    ModuleErrorInfo,
    ModuleListResult,
    ModuleUpdateParameters,
    NodeCount,
    NodeCountProperties,
    NodeCounts,
    NonAzureQueryProperties,
    Operation,
    OperationDisplay,
    OperationListResult,
    ProxyResource,
    PythonPackageCreateParameters,
    PythonPackageUpdateParameters,
    Resource,
    RunAsCredentialAssociationProperty,
    Runbook,
    RunbookAssociationProperty,
    RunbookCreateOrUpdateDraftParameters,
    RunbookCreateOrUpdateDraftProperties,
    RunbookCreateOrUpdateParameters,
    RunbookDraft,
    RunbookDraftUndoEditResult,
    RunbookListResult,
    RunbookParameter,
    RunbookUpdateParameters,
    Schedule,
    ScheduleAssociationProperty,
    ScheduleCreateOrUpdateParameters,
    ScheduleListResult,
    ScheduleProperties,
    ScheduleUpdateParameters,
    Sku,
    SoftareUpdateConfigurationRunTaskProperties,
    SoftareUpdateConfigurationRunTasks,
    SoftwareUpdateConfiguration,
    SoftwareUpdateConfigurationCollectionItem,
    SoftwareUpdateConfigurationListResult,
    SoftwareUpdateConfigurationMachineRun,
    SoftwareUpdateConfigurationMachineRunListResult,
    SoftwareUpdateConfigurationRun,
    SoftwareUpdateConfigurationRunListResult,
    SoftwareUpdateConfigurationTasks,
    SourceControl,
    SourceControlCreateOrUpdateParameters,
    SourceControlListResult,
    SourceControlSecurityTokenProperties,
    SourceControlSyncJob,
    SourceControlSyncJobById,
    SourceControlSyncJobCreateParameters,
    SourceControlSyncJobListResult,
    SourceControlSyncJobStream,
    SourceControlSyncJobStreamById,
    SourceControlSyncJobStreamsListBySyncJob,
    SourceControlUpdateParameters,
    Statistics,
    StatisticsListResult,
    TagSettingsProperties,
    TargetProperties,
    TaskProperties,
    TestJob,
    TestJobCreateParameters,
    TrackedResource,
    TypeField,
    TypeFieldListResult,
    UpdateConfiguration,
    UpdateConfigurationNavigation,
    Usage,
    UsageCounterName,
    UsageListResult,
    Variable,
    VariableCreateOrUpdateParameters,
    VariableListResult,
    VariableUpdateParameters,
    Watcher,
    WatcherListResult,
    WatcherUpdateParameters,
    Webhook,
    WebhookCreateOrUpdateParameters,
    WebhookListResult,
    WebhookUpdateParameters,
    WindowsProperties,
)

from ._automation_client_enums import (  # type: ignore
    AgentRegistrationKeyName,
    AutomationAccountState,
    AutomationKeyName,
    AutomationKeyPermissions,
    ContentSourceType,
    CountType,
    DscConfigurationState,
    GroupTypeEnum,
    HttpStatusCode,
    JobProvisioningState,
    JobStatus,
    JobStreamType,
    LinuxUpdateClasses,
    ModuleProvisioningState,
    OperatingSystemType,
    ProvisioningState,
    RunbookState,
    RunbookTypeEnum,
    ScheduleDay,
    ScheduleFrequency,
    SkuNameEnum,
    SourceType,
    StreamType,
    SyncType,
    TagOperators,
    TokenType,
    WindowsUpdateClasses,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Activity",
    "ActivityListResult",
    "ActivityOutputType",
    "ActivityParameter",
    "ActivityParameterSet",
    "ActivityParameterValidationSet",
    "AdvancedSchedule",
    "AdvancedScheduleMonthlyOccurrence",
    "AgentRegistration",
    "AgentRegistrationKeys",
    "AgentRegistrationRegenerateKeyParameter",
    "AutomationAccount",
    "AutomationAccountCreateOrUpdateParameters",
    "AutomationAccountListResult",
    "AutomationAccountUpdateParameters",
    "AzureQueryProperties",
    "Certificate",
    "CertificateCreateOrUpdateParameters",
    "CertificateListResult",
    "CertificateUpdateParameters",
    "CollectionItemUpdateConfiguration",
    "Connection",
    "ConnectionCreateOrUpdateParameters",
    "ConnectionListResult",
    "ConnectionType",
    "ConnectionTypeAssociationProperty",
    "ConnectionTypeCreateOrUpdateParameters",
    "ConnectionTypeListResult",
    "ConnectionUpdateParameters",
    "ContentHash",
    "ContentLink",
    "ContentSource",
    "Credential",
    "CredentialCreateOrUpdateParameters",
    "CredentialListResult",
    "CredentialUpdateParameters",
    "DscCompilationJob",
    "DscCompilationJobCreateParameters",
    "DscCompilationJobListResult",
    "DscConfiguration",
    "DscConfigurationAssociationProperty",
    "DscConfigurationCreateOrUpdateParameters",
    "DscConfigurationListResult",
    "DscConfigurationParameter",
    "DscConfigurationUpdateParameters",
    "DscMetaConfiguration",
    "DscNode",
    "DscNodeConfiguration",
    "DscNodeConfigurationCreateOrUpdateParameters",
    "DscNodeConfigurationListResult",
    "DscNodeExtensionHandlerAssociationProperty",
    "DscNodeListResult",
    "DscNodeReport",
    "DscNodeReportListResult",
    "DscNodeUpdateParameters",
    "DscNodeUpdateParametersProperties",
    "DscReportError",
    "DscReportResource",
    "DscReportResourceNavigation",
    "ErrorResponse",
    "FieldDefinition",
    "HybridRunbookWorker",
    "HybridRunbookWorkerGroup",
    "HybridRunbookWorkerGroupUpdateParameters",
    "HybridRunbookWorkerGroupsListResult",
    "Job",
    "JobCollectionItem",
    "JobCreateParameters",
    "JobListResultV2",
    "JobNavigation",
    "JobSchedule",
    "JobScheduleCreateParameters",
    "JobScheduleListResult",
    "JobStream",
    "JobStreamListResult",
    "Key",
    "KeyListResult",
    "LinkedWorkspace",
    "LinuxProperties",
    "Module",
    "ModuleCreateOrUpdateParameters",
    "ModuleErrorInfo",
    "ModuleListResult",
    "ModuleUpdateParameters",
    "NodeCount",
    "NodeCountProperties",
    "NodeCounts",
    "NonAzureQueryProperties",
    "Operation",
    "OperationDisplay",
    "OperationListResult",
    "ProxyResource",
    "PythonPackageCreateParameters",
    "PythonPackageUpdateParameters",
    "Resource",
    "RunAsCredentialAssociationProperty",
    "Runbook",
    "RunbookAssociationProperty",
    "RunbookCreateOrUpdateDraftParameters",
    "RunbookCreateOrUpdateDraftProperties",
    "RunbookCreateOrUpdateParameters",
    "RunbookDraft",
    "RunbookDraftUndoEditResult",
    "RunbookListResult",
    "RunbookParameter",
    "RunbookUpdateParameters",
    "Schedule",
    "ScheduleAssociationProperty",
    "ScheduleCreateOrUpdateParameters",
    "ScheduleListResult",
    "ScheduleProperties",
    "ScheduleUpdateParameters",
    "Sku",
    "SoftareUpdateConfigurationRunTaskProperties",
    "SoftareUpdateConfigurationRunTasks",
    "SoftwareUpdateConfiguration",
    "SoftwareUpdateConfigurationCollectionItem",
    "SoftwareUpdateConfigurationListResult",
    "SoftwareUpdateConfigurationMachineRun",
    "SoftwareUpdateConfigurationMachineRunListResult",
    "SoftwareUpdateConfigurationRun",
    "SoftwareUpdateConfigurationRunListResult",
    "SoftwareUpdateConfigurationTasks",
    "SourceControl",
    "SourceControlCreateOrUpdateParameters",
    "SourceControlListResult",
    "SourceControlSecurityTokenProperties",
    "SourceControlSyncJob",
    "SourceControlSyncJobById",
    "SourceControlSyncJobCreateParameters",
    "SourceControlSyncJobListResult",
    "SourceControlSyncJobStream",
    "SourceControlSyncJobStreamById",
    "SourceControlSyncJobStreamsListBySyncJob",
    "SourceControlUpdateParameters",
    "Statistics",
    "StatisticsListResult",
    "TagSettingsProperties",
    "TargetProperties",
    "TaskProperties",
    "TestJob",
    "TestJobCreateParameters",
    "TrackedResource",
    "TypeField",
    "TypeFieldListResult",
    "UpdateConfiguration",
    "UpdateConfigurationNavigation",
    "Usage",
    "UsageCounterName",
    "UsageListResult",
    "Variable",
    "VariableCreateOrUpdateParameters",
    "VariableListResult",
    "VariableUpdateParameters",
    "Watcher",
    "WatcherListResult",
    "WatcherUpdateParameters",
    "Webhook",
    "WebhookCreateOrUpdateParameters",
    "WebhookListResult",
    "WebhookUpdateParameters",
    "WindowsProperties",
    "AgentRegistrationKeyName",
    "AutomationAccountState",
    "AutomationKeyName",
    "AutomationKeyPermissions",
    "ContentSourceType",
    "CountType",
    "DscConfigurationState",
    "GroupTypeEnum",
    "HttpStatusCode",
    "JobProvisioningState",
    "JobStatus",
    "JobStreamType",
    "LinuxUpdateClasses",
    "ModuleProvisioningState",
    "OperatingSystemType",
    "ProvisioningState",
    "RunbookState",
    "RunbookTypeEnum",
    "ScheduleDay",
    "ScheduleFrequency",
    "SkuNameEnum",
    "SourceType",
    "StreamType",
    "SyncType",
    "TagOperators",
    "TokenType",
    "WindowsUpdateClasses",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
