# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.apimanagement.aio import ApiManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApiManagementGatewayCertificateAuthorityOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApiManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_gateway_certificate_authority_get(self, resource_group):
        response = await self.client.gateway_certificate_authority.get(
            resource_group_name=resource_group.name,
            service_name="str",
            gateway_id="str",
            certificate_id="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_gateway_certificate_authority_get_entity_tag(self, resource_group):
        response = await self.client.gateway_certificate_authority.get_entity_tag(
            resource_group_name=resource_group.name,
            service_name="str",
            gateway_id="str",
            certificate_id="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_gateway_certificate_authority_create_or_update(self, resource_group):
        response = await self.client.gateway_certificate_authority.create_or_update(
            resource_group_name=resource_group.name,
            service_name="str",
            gateway_id="str",
            certificate_id="str",
            parameters={
                "id": "str",
                "name": "str",
                "properties": {"isTrusted": bool},
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_gateway_certificate_authority_delete(self, resource_group):
        response = await self.client.gateway_certificate_authority.delete(
            resource_group_name=resource_group.name,
            service_name="str",
            gateway_id="str",
            certificate_id="str",
            etag="str",
            match_condition=~azure.core.MatchConditions,
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_gateway_certificate_authority_list_by_service(self, resource_group):
        response = self.client.gateway_certificate_authority.list_by_service(
            resource_group_name=resource_group.name,
            service_name="str",
            gateway_id="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...
