# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.compute import ComputeManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-compute
# USAGE
    python gallery_add_to_sharing_profile.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ComputeManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.gallery_sharing_profile.begin_update(
        resource_group_name="myResourceGroup",
        gallery_name="myGalleryName",
        sharing_update={
            "groups": [
                {
                    "ids": ["34a4ab42-0d72-47d9-bd1a-aed207386dac", "380fd389-260b-41aa-bad9-0a83108c370b"],
                    "type": "Subscriptions",
                },
                {"ids": ["c24c76aa-8897-4027-9b03-8f7928b54ff6"], "type": "AADTenants"},
            ],
            "operationType": "Add",
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-03-03/galleryExamples/Gallery_AddToSharingProfile.json
if __name__ == "__main__":
    main()
