# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.chaos import ChaosManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-chaos
# USAGE
    python scenario_configurations_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ChaosManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.scenario_configurations.begin_create_or_update(
        resource_group_name="exampleRG",
        workspace_name="exampleWorkspace",
        scenario_name="12345678-1234-1234-1234-123456789012",
        scenario_configuration_name="config-5678-9012-3456-789012345678",
        resource={
            "properties": {
                "exclusions": {
                    "resources": [
                        "/subscriptions/6b052e15-03d3-4f17-b2e1-be7f07588291/resourceGroups/exampleRG/providers/Microsoft.Compute/virtualMachines/protectedVM"
                    ],
                    "tags": [{"key": "environment", "value": "production"}],
                    "types": ["Microsoft.Compute/virtualMachineScaleSets"],
                },
                "filters": {"locations": ["eastus"], "zones": ["1"]},
                "parameters": [
                    {"key": "duration", "value": "PT10M"},
                    {
                        "key": "targetResourceIds",
                        "value": '["/subscriptions/6b052e15-03d3-4f17-b2e1-be7f07588291/resourceGroups/exampleRG/providers/Microsoft.Compute/virtualMachines/vm1","/subscriptions/6b052e15-03d3-4f17-b2e1-be7f07588291/resourceGroups/exampleRG/providers/Microsoft.Compute/virtualMachines/vm2"]',
                    },
                ],
                "scenarioId": "/subscriptions/6b052e15-03d3-4f17-b2e1-be7f07588291/resourceGroups/exampleRG/providers/Microsoft.Chaos/workspaces/exampleWorkspace/scenarios/12345678-1234-1234-1234-123456789012",
            }
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-05-01-preview/ScenarioConfigurations_CreateOrUpdate.json
if __name__ == "__main__":
    main()
