# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.billingbenefits import BillingBenefitsMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-billingbenefits
# USAGE
    python conditional_credit_validate_create_contributor.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BillingBenefitsMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.benefit.validate(
        body={
            "benefits": [
                {
                    "benefitType": "ConditionalCredits",
                    "properties": {
                        "displayName": "Contributor CACO Test",
                        "entityType": "Contributor",
                        "primaryResourceId": "/subscriptions/eef82110-c91b-4395-9420-fcfcbefc5a47/resourceGroups/primaryRG/providers/Microsoft.BillingBenefits/conditionalCredits/primaryCACO",
                        "productCode": "000187f7-0000-0260-ab43-b8473ce57f1d",
                        "startAt": "2025-09-01T00:00:00Z",
                    },
                }
            ]
        },
    )
    print(response)


# x-ms-original-file: 2025-12-01-preview/ConditionalCreditValidateCreateContributor.json
if __name__ == "__main__":
    main()
