# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.costmanagement.aio import CostManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCostManagementViewsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CostManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_views_get(self, resource_group):
        response = await self.client.views.get(
            view_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_views_create_or_update(self, resource_group):
        response = await self.client.views.create_or_update(
            view_name="str",
            parameters={
                "eTag": "str",
                "id": "str",
                "name": "str",
                "properties": {
                    "accumulated": "str",
                    "chart": "str",
                    "createdOn": "2020-02-20 00:00:00",
                    "currency": "str",
                    "dateRange": "str",
                    "displayName": "str",
                    "kpis": [{"enabled": bool, "id": "str", "type": "str"}],
                    "metric": "str",
                    "modifiedOn": "2020-02-20 00:00:00",
                    "pivots": [{"name": "str", "type": "str"}],
                    "query": {
                        "timeframe": "str",
                        "type": "str",
                        "dataSet": {
                            "aggregation": {"str": {"function": "str", "name": "str"}},
                            "configuration": {"columns": ["str"]},
                            "filter": {
                                "and": [...],
                                "dimensions": {"name": "str", "operator": "str", "values": ["str"]},
                                "or": [...],
                                "tags": {"name": "str", "operator": "str", "values": ["str"]},
                            },
                            "granularity": "str",
                            "grouping": [{"name": "str", "type": "str"}],
                            "sorting": [{"name": "str", "direction": "str"}],
                        },
                        "includeMonetaryCommitment": bool,
                        "timePeriod": {"from": "2020-02-20 00:00:00", "to": "2020-02-20 00:00:00"},
                    },
                    "scope": "str",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_views_delete(self, resource_group):
        response = await self.client.views.delete(
            view_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_views_list(self, resource_group):
        response = self.client.views.list()
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_views_get_by_scope(self, resource_group):
        response = await self.client.views.get_by_scope(
            scope="str",
            view_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_views_create_or_update_by_scope(self, resource_group):
        response = await self.client.views.create_or_update_by_scope(
            scope="str",
            view_name="str",
            parameters={
                "eTag": "str",
                "id": "str",
                "name": "str",
                "properties": {
                    "accumulated": "str",
                    "chart": "str",
                    "createdOn": "2020-02-20 00:00:00",
                    "currency": "str",
                    "dateRange": "str",
                    "displayName": "str",
                    "kpis": [{"enabled": bool, "id": "str", "type": "str"}],
                    "metric": "str",
                    "modifiedOn": "2020-02-20 00:00:00",
                    "pivots": [{"name": "str", "type": "str"}],
                    "query": {
                        "timeframe": "str",
                        "type": "str",
                        "dataSet": {
                            "aggregation": {"str": {"function": "str", "name": "str"}},
                            "configuration": {"columns": ["str"]},
                            "filter": {
                                "and": [...],
                                "dimensions": {"name": "str", "operator": "str", "values": ["str"]},
                                "or": [...],
                                "tags": {"name": "str", "operator": "str", "values": ["str"]},
                            },
                            "granularity": "str",
                            "grouping": [{"name": "str", "type": "str"}],
                            "sorting": [{"name": "str", "direction": "str"}],
                        },
                        "includeMonetaryCommitment": bool,
                        "timePeriod": {"from": "2020-02-20 00:00:00", "to": "2020-02-20 00:00:00"},
                    },
                    "scope": "str",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_views_delete_by_scope(self, resource_group):
        response = await self.client.views.delete_by_scope(
            scope="str",
            view_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_views_list_by_scope(self, resource_group):
        response = self.client.views.list_by_scope(
            scope="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...
