# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    APIKeyRequest,
    Annotation,
    AnnotationError,
    ApplicationInsightsComponent,
    ApplicationInsightsComponentAPIKey,
    ApplicationInsightsComponentAnalyticsItem,
    ApplicationInsightsComponentAnalyticsItemProperties,
    ApplicationInsightsComponentAvailableFeatures,
    ApplicationInsightsComponentBillingFeatures,
    ApplicationInsightsComponentDataVolumeCap,
    ApplicationInsightsComponentExportConfiguration,
    ApplicationInsightsComponentExportRequest,
    ApplicationInsightsComponentFavorite,
    ApplicationInsightsComponentFeature,
    ApplicationInsightsComponentFeatureCapabilities,
    ApplicationInsightsComponentFeatureCapability,
    ApplicationInsightsComponentProactiveDetectionConfiguration,
    ApplicationInsightsComponentProactiveDetectionConfigurationRuleDefinitions,
    ApplicationInsightsComponentProperties,
    ApplicationInsightsComponentQuotaStatus,
    ApplicationInsightsComponentWebTestLocation,
    ComponentLinkedStorageAccounts,
    ComponentLinkedStorageAccountsPatch,
    ComponentPurgeBody,
    ComponentPurgeBodyFilters,
    ComponentPurgeResponse,
    ComponentPurgeStatusResponse,
    ComponentsResource,
    DeletedWorkbook,
    DeletedWorkbookError,
    DeletedWorkbookErrorDefinition,
    DeletedWorkbookInnerErrorTrace,
    DeletedWorkbookProperties,
    DeletedWorkbookResource,
    ErrorFieldContract,
    ErrorResponse,
    ErrorResponseComponents,
    ErrorResponseComponentsError,
    ErrorResponseLinkedStorage,
    ErrorResponseLinkedStorageError,
    HeaderField,
    InnerError,
    LinkedStorageAccountsProperties,
    LiveTokenResponse,
    ManagedServiceIdentity,
    Operation,
    OperationDisplay,
    PrivateLinkScopedResource,
    ProxyResource,
    Resource,
    SystemData,
    TagsResource,
    TrackedResource,
    UserAssignedIdentity,
    WebTest,
    WebTestGeolocation,
    WebTestProperties,
    WebTestPropertiesConfiguration,
    WebTestPropertiesRequest,
    WebTestPropertiesValidationRules,
    WebTestPropertiesValidationRulesContentValidation,
    WebtestsResource,
    WorkItemConfiguration,
    WorkItemConfigurationError,
    WorkItemCreateConfiguration,
    Workbook,
    WorkbookError,
    WorkbookErrorDefinition,
    WorkbookInnerErrorTrace,
    WorkbookProperties,
    WorkbookPropertiesUpdateParameters,
    WorkbookResourceIdentity,
    WorkbookTemplate,
    WorkbookTemplateError,
    WorkbookTemplateErrorBody,
    WorkbookTemplateErrorFieldContract,
    WorkbookTemplateGallery,
    WorkbookTemplateLocalizedGallery,
    WorkbookTemplateProperties,
    WorkbookTemplateUpdateParameters,
    WorkbookUpdateParameters,
)

from ._enums import (  # type: ignore
    ApplicationType,
    CategoryType,
    CreatedByType,
    FavoriteSourceType,
    FavoriteType,
    FlowType,
    IngestionMode,
    ItemScope,
    ItemScopePath,
    ItemType,
    ItemTypeParameter,
    ManagedServiceIdentityType,
    PublicNetworkAccessType,
    PurgeState,
    RequestSource,
    StorageType,
    WebTestKind,
    WorkbookSharedTypeKind,
    WorkbookUpdateSharedTypeKind,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "APIKeyRequest",
    "Annotation",
    "AnnotationError",
    "ApplicationInsightsComponent",
    "ApplicationInsightsComponentAPIKey",
    "ApplicationInsightsComponentAnalyticsItem",
    "ApplicationInsightsComponentAnalyticsItemProperties",
    "ApplicationInsightsComponentAvailableFeatures",
    "ApplicationInsightsComponentBillingFeatures",
    "ApplicationInsightsComponentDataVolumeCap",
    "ApplicationInsightsComponentExportConfiguration",
    "ApplicationInsightsComponentExportRequest",
    "ApplicationInsightsComponentFavorite",
    "ApplicationInsightsComponentFeature",
    "ApplicationInsightsComponentFeatureCapabilities",
    "ApplicationInsightsComponentFeatureCapability",
    "ApplicationInsightsComponentProactiveDetectionConfiguration",
    "ApplicationInsightsComponentProactiveDetectionConfigurationRuleDefinitions",
    "ApplicationInsightsComponentProperties",
    "ApplicationInsightsComponentQuotaStatus",
    "ApplicationInsightsComponentWebTestLocation",
    "ComponentLinkedStorageAccounts",
    "ComponentLinkedStorageAccountsPatch",
    "ComponentPurgeBody",
    "ComponentPurgeBodyFilters",
    "ComponentPurgeResponse",
    "ComponentPurgeStatusResponse",
    "ComponentsResource",
    "DeletedWorkbook",
    "DeletedWorkbookError",
    "DeletedWorkbookErrorDefinition",
    "DeletedWorkbookInnerErrorTrace",
    "DeletedWorkbookProperties",
    "DeletedWorkbookResource",
    "ErrorFieldContract",
    "ErrorResponse",
    "ErrorResponseComponents",
    "ErrorResponseComponentsError",
    "ErrorResponseLinkedStorage",
    "ErrorResponseLinkedStorageError",
    "HeaderField",
    "InnerError",
    "LinkedStorageAccountsProperties",
    "LiveTokenResponse",
    "ManagedServiceIdentity",
    "Operation",
    "OperationDisplay",
    "PrivateLinkScopedResource",
    "ProxyResource",
    "Resource",
    "SystemData",
    "TagsResource",
    "TrackedResource",
    "UserAssignedIdentity",
    "WebTest",
    "WebTestGeolocation",
    "WebTestProperties",
    "WebTestPropertiesConfiguration",
    "WebTestPropertiesRequest",
    "WebTestPropertiesValidationRules",
    "WebTestPropertiesValidationRulesContentValidation",
    "WebtestsResource",
    "WorkItemConfiguration",
    "WorkItemConfigurationError",
    "WorkItemCreateConfiguration",
    "Workbook",
    "WorkbookError",
    "WorkbookErrorDefinition",
    "WorkbookInnerErrorTrace",
    "WorkbookProperties",
    "WorkbookPropertiesUpdateParameters",
    "WorkbookResourceIdentity",
    "WorkbookTemplate",
    "WorkbookTemplateError",
    "WorkbookTemplateErrorBody",
    "WorkbookTemplateErrorFieldContract",
    "WorkbookTemplateGallery",
    "WorkbookTemplateLocalizedGallery",
    "WorkbookTemplateProperties",
    "WorkbookTemplateUpdateParameters",
    "WorkbookUpdateParameters",
    "ApplicationType",
    "CategoryType",
    "CreatedByType",
    "FavoriteSourceType",
    "FavoriteType",
    "FlowType",
    "IngestionMode",
    "ItemScope",
    "ItemScopePath",
    "ItemType",
    "ItemTypeParameter",
    "ManagedServiceIdentityType",
    "PublicNetworkAccessType",
    "PurgeState",
    "RequestSource",
    "StorageType",
    "WebTestKind",
    "WorkbookSharedTypeKind",
    "WorkbookUpdateSharedTypeKind",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
