# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Extensible enum. Indicates the action type. "Internal" refers to actions that are for internal
    only APIs.
    """

    INTERNAL = "Internal"
    """Actions are for internal-only APIs."""


class ClusterType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AppLinkMember cluster type."""

    AKS = "AKS"
    """Azure Kubernetes Service."""


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The kind of entity that created the resource."""

    USER = "User"
    """The entity was created by a user."""
    APPLICATION = "Application"
    """The entity was created by an application."""
    MANAGED_IDENTITY = "ManagedIdentity"
    """The entity was created by a managed identity."""
    KEY = "Key"
    """The entity was created by a key."""


class EastWestGatewayVisibility(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """East-West gateway visibility."""

    INTERNAL = "Internal"
    """Use an internal load balancer for the east-west gateway."""
    EXTERNAL = "External"
    """Use an external load balancer for the east-west gateway."""


class ManagedServiceIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of managed service identity (where both SystemAssigned and UserAssigned types are
    allowed).
    """

    NONE = "None"
    """No managed identity."""
    SYSTEM_ASSIGNED = "SystemAssigned"
    """System assigned managed identity."""
    USER_ASSIGNED = "UserAssigned"
    """User assigned managed identity."""
    SYSTEM_ASSIGNED_USER_ASSIGNED = "SystemAssigned,UserAssigned"
    """System and user assigned managed identity."""


class Origin(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit
    logs UX. Default value is "user,system".
    """

    USER = "user"
    """Indicates the operation is initiated by a user."""
    SYSTEM = "system"
    """Indicates the operation is initiated by a system."""
    USER_SYSTEM = "user,system"
    """Indicates the operation is initiated by a user or system."""


class ProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Provisioning state of the resource."""

    SUCCEEDED = "Succeeded"
    """Resource has been created."""
    FAILED = "Failed"
    """Resource creation failed."""
    CANCELED = "Canceled"
    """Resource creation was canceled."""
    PROVISIONING = "Provisioning"
    """Resource is getting provisioned."""
    UPDATING = "Updating"
    """Resource is Updating."""
    DELETING = "Deleting"
    """Resource is Deleting."""
    ACCEPTED = "Accepted"
    """Resource has been Accepted."""


class UpgradeMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AppLinkMember upgrade mode."""

    FULLY_MANAGED = "FullyManaged"
    """Fully managed upgrade mode."""
    SELF_MANAGED = "SelfManaged"
    """Self managed upgrade mode."""


class UpgradeReleaseChannel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AppLinkMember upgrade release channel."""

    RAPID = "Rapid"
    """Rapid release channel."""
    STABLE = "Stable"
    """Stable release channel."""
