# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.compute import ComputeManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-compute
# USAGE
    python gallery_image_version_create_with_additional_replica_sets.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ComputeManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.gallery_image_versions.begin_create_or_update(
        resource_group_name="myResourceGroup",
        gallery_name="myGalleryName",
        gallery_image_name="myGalleryImageName",
        gallery_image_version_name="1.0.0",
        gallery_image_version={
            "location": "West US",
            "properties": {
                "publishingProfile": {
                    "targetRegions": [
                        {
                            "additionalReplicaSets": [
                                {"regionalReplicaCount": 1, "storageAccountType": "PreviumV2_LRS"}
                            ],
                            "encryption": {
                                "dataDiskImages": [
                                    {
                                        "diskEncryptionSetId": "/subscriptions/{subscriptionId}/resourceGroups/myResourceGroup/providers/Microsoft.Compute/diskEncryptionSet/myOtherWestUSDiskEncryptionSet",
                                        "lun": 0,
                                    },
                                    {
                                        "diskEncryptionSetId": "/subscriptions/{subscriptionId}/resourceGroups/myResourceGroup/providers/Microsoft.Compute/diskEncryptionSet/myWestUSDiskEncryptionSet",
                                        "lun": 1,
                                    },
                                ],
                                "osDiskImage": {
                                    "diskEncryptionSetId": "/subscriptions/{subscriptionId}/resourceGroups/myResourceGroup/providers/Microsoft.Compute/diskEncryptionSet/myWestUSDiskEncryptionSet"
                                },
                            },
                            "excludeFromLatest": False,
                            "name": "West US",
                            "regionalReplicaCount": 1,
                        },
                        {
                            "encryption": {
                                "dataDiskImages": [
                                    {
                                        "diskEncryptionSetId": "/subscriptions/{subscriptionId}/resourceGroups/myResourceGroup/providers/Microsoft.Compute/diskEncryptionSet/myOtherEastUSDiskEncryptionSet",
                                        "lun": 0,
                                    },
                                    {
                                        "diskEncryptionSetId": "/subscriptions/{subscriptionId}/resourceGroups/myResourceGroup/providers/Microsoft.Compute/diskEncryptionSet/myEastUSDiskEncryptionSet",
                                        "lun": 1,
                                    },
                                ],
                                "osDiskImage": {
                                    "diskEncryptionSetId": "/subscriptions/{subscriptionId}/resourceGroups/myResourceGroup/providers/Microsoft.Compute/diskEncryptionSet/myEastUSDiskEncryptionSet"
                                },
                            },
                            "excludeFromLatest": False,
                            "name": "East US",
                            "regionalReplicaCount": 2,
                            "storageAccountType": "Standard_ZRS",
                        },
                    ]
                },
                "safetyProfile": {"allowDeletionOfReplicatedLocations": False},
                "storageProfile": {
                    "source": {
                        "id": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroup}/providers/Microsoft.Compute/images/{imageName}"
                    }
                },
            },
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-03-03/galleryExamples/GalleryImageVersion_Create_WithAdditionalReplicaSets.json
if __name__ == "__main__":
    main()
