# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Operations  # type: ignore
from ._operations import FactoriesOperations  # type: ignore
from ._operations import ExposureControlOperations  # type: ignore
from ._operations import PipelineRunsOperations  # type: ignore
from ._operations import ActivityRunsOperations  # type: ignore
from ._operations import TriggersOperations  # type: ignore
from ._operations import TriggerRunsOperations  # type: ignore
from ._operations import DataFlowDebugSessionOperations  # type: ignore
from ._operations import PrivateLinkResourcesOperations  # type: ignore
from ._operations import IntegrationRuntimesOperations  # type: ignore
from ._operations import IntegrationRuntimeObjectMetadataOperations  # type: ignore
from ._operations import IntegrationRuntimeNodesOperations  # type: ignore
from ._operations import IntegrationRuntimeOperations  # type: ignore
from ._operations import LinkedServicesOperations  # type: ignore
from ._operations import DatasetsOperations  # type: ignore
from ._operations import PipelinesOperations  # type: ignore
from ._operations import DataFlowsOperations  # type: ignore
from ._operations import ManagedVirtualNetworksOperations  # type: ignore
from ._operations import ManagedPrivateEndpointsOperations  # type: ignore
from ._operations import CredentialOperationsOperations  # type: ignore
from ._operations import PrivateEndpointConnectionOperations  # type: ignore
from ._operations import PrivateEndPointConnectionsOperations  # type: ignore
from ._operations import GlobalParametersOperations  # type: ignore
from ._operations import ChangeDataCaptureOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "FactoriesOperations",
    "ExposureControlOperations",
    "PipelineRunsOperations",
    "ActivityRunsOperations",
    "TriggersOperations",
    "TriggerRunsOperations",
    "DataFlowDebugSessionOperations",
    "PrivateLinkResourcesOperations",
    "IntegrationRuntimesOperations",
    "IntegrationRuntimeObjectMetadataOperations",
    "IntegrationRuntimeNodesOperations",
    "IntegrationRuntimeOperations",
    "LinkedServicesOperations",
    "DatasetsOperations",
    "PipelinesOperations",
    "DataFlowsOperations",
    "ManagedVirtualNetworksOperations",
    "ManagedPrivateEndpointsOperations",
    "CredentialOperationsOperations",
    "PrivateEndpointConnectionOperations",
    "PrivateEndPointConnectionsOperations",
    "GlobalParametersOperations",
    "ChangeDataCaptureOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
