# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.cdn.aio import CdnManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCdnManagementPoliciesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CdnManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_get(self, resource_group):
        response = await self.client.policies.get(
            resource_group_name=resource_group.name,
            policy_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_begin_create_or_update(self, resource_group):
        response = await (
            await self.client.policies.begin_create_or_update(
                resource_group_name=resource_group.name,
                policy_name="str",
                cdn_web_application_firewall_policy={
                    "location": "str",
                    "sku": {"name": "str"},
                    "etag": "str",
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "customRules": {
                            "rules": [
                                {
                                    "action": "str",
                                    "matchConditions": [
                                        {
                                            "matchValue": ["str"],
                                            "matchVariable": "str",
                                            "operator": "str",
                                            "negateCondition": bool,
                                            "selector": "str",
                                            "transforms": ["str"],
                                        }
                                    ],
                                    "name": "str",
                                    "priority": 0,
                                    "enabledState": "str",
                                }
                            ]
                        },
                        "endpointLinks": [{"id": "str"}],
                        "extendedProperties": {"str": "str"},
                        "managedRules": {
                            "managedRuleSets": [
                                {
                                    "ruleSetType": "str",
                                    "ruleSetVersion": "str",
                                    "anomalyScore": 0,
                                    "ruleGroupOverrides": [
                                        {
                                            "ruleGroupName": "str",
                                            "rules": [{"ruleId": "str", "action": "str", "enabledState": "str"}],
                                        }
                                    ],
                                }
                            ]
                        },
                        "policySettings": {
                            "defaultCustomBlockResponseBody": "str",
                            "defaultCustomBlockResponseStatusCode": 0,
                            "defaultRedirectUrl": "str",
                            "enabledState": "str",
                            "mode": "str",
                        },
                        "provisioningState": "str",
                        "rateLimitRules": {
                            "rules": [
                                {
                                    "action": "str",
                                    "matchConditions": [
                                        {
                                            "matchValue": ["str"],
                                            "matchVariable": "str",
                                            "operator": "str",
                                            "negateCondition": bool,
                                            "selector": "str",
                                            "transforms": ["str"],
                                        }
                                    ],
                                    "name": "str",
                                    "priority": 0,
                                    "rateLimitDurationInMinutes": 0,
                                    "rateLimitThreshold": 0,
                                    "enabledState": "str",
                                }
                            ]
                        },
                        "resourceState": "str",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_begin_update(self, resource_group):
        response = await (
            await self.client.policies.begin_update(
                resource_group_name=resource_group.name,
                policy_name="str",
                cdn_web_application_firewall_policy_patch_parameters={"tags": {"str": "str"}},
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_delete(self, resource_group):
        response = await self.client.policies.delete(
            resource_group_name=resource_group.name,
            policy_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_policies_list(self, resource_group):
        response = self.client.policies.list(
            resource_group_name=resource_group.name,
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...
