# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Operations  # type: ignore
from ._operations import AgreementsOperations  # type: ignore
from ._operations import BillingRequestsOperations  # type: ignore
from ._operations import BillingRoleAssignmentsOperations  # type: ignore
from ._operations import InvoicesOperations  # type: ignore
from ._operations import ReservationsOperations  # type: ignore
from ._operations import BillingAccountsOperations  # type: ignore
from ._operations import AssociatedTenantsOperations  # type: ignore
from ._operations import AvailableBalancesOperations  # type: ignore
from ._operations import ProductsOperations  # type: ignore
from ._operations import BillingProfilesOperations  # type: ignore
from ._operations import BillingSubscriptionsOperations  # type: ignore
from ._operations import PoliciesOperations  # type: ignore
from ._operations import CustomersOperations  # type: ignore
from ._operations import DepartmentsOperations  # type: ignore
from ._operations import EnrollmentAccountsOperations  # type: ignore
from ._operations import InvoiceSectionsOperations  # type: ignore
from ._operations import PaymentMethodsOperations  # type: ignore
from ._operations import ReservationOrdersOperations  # type: ignore
from ._operations import TransfersOperations  # type: ignore
from ._operations import PartnerTransfersOperations  # type: ignore
from ._operations import RecipientTransfersOperations  # type: ignore
from ._operations import TransactionsOperations  # type: ignore
from ._operations import BillingPermissionsOperations  # type: ignore
from ._operations import SavingsPlansOperations  # type: ignore
from ._operations import BillingPropertyOperations  # type: ignore
from ._operations import BillingRoleDefinitionOperations  # type: ignore
from ._operations import SavingsPlanOrdersOperations  # type: ignore
from ._operations import BillingSubscriptionsAliasesOperations  # type: ignore
from ._operations import AddressOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "AgreementsOperations",
    "BillingRequestsOperations",
    "BillingRoleAssignmentsOperations",
    "InvoicesOperations",
    "ReservationsOperations",
    "BillingAccountsOperations",
    "AssociatedTenantsOperations",
    "AvailableBalancesOperations",
    "ProductsOperations",
    "BillingProfilesOperations",
    "BillingSubscriptionsOperations",
    "PoliciesOperations",
    "CustomersOperations",
    "DepartmentsOperations",
    "EnrollmentAccountsOperations",
    "InvoiceSectionsOperations",
    "PaymentMethodsOperations",
    "ReservationOrdersOperations",
    "TransfersOperations",
    "PartnerTransfersOperations",
    "RecipientTransfersOperations",
    "TransactionsOperations",
    "BillingPermissionsOperations",
    "SavingsPlansOperations",
    "BillingPropertyOperations",
    "BillingRoleDefinitionOperations",
    "SavingsPlanOrdersOperations",
    "BillingSubscriptionsAliasesOperations",
    "AddressOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
