# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.apimanagement.aio import ApiManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApiManagementWorkspaceApiOperationOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApiManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_api_operation_get(self, resource_group):
        response = await self.client.workspace_api_operation.get(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            api_id="str",
            operation_id="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_api_operation_get_entity_tag(self, resource_group):
        response = await self.client.workspace_api_operation.get_entity_tag(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            api_id="str",
            operation_id="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_api_operation_create_or_update(self, resource_group):
        response = await self.client.workspace_api_operation.create_or_update(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            api_id="str",
            operation_id="str",
            parameters={
                "id": "str",
                "name": "str",
                "properties": {
                    "displayName": "str",
                    "method": "str",
                    "urlTemplate": "str",
                    "description": "str",
                    "policies": "str",
                    "request": {
                        "description": "str",
                        "headers": [
                            {
                                "name": "str",
                                "type": "str",
                                "defaultValue": "str",
                                "description": "str",
                                "examples": {
                                    "str": {"description": "str", "externalValue": "str", "summary": "str", "value": {}}
                                },
                                "required": bool,
                                "schemaId": "str",
                                "typeName": "str",
                                "values": ["str"],
                            }
                        ],
                        "queryParameters": [
                            {
                                "name": "str",
                                "type": "str",
                                "defaultValue": "str",
                                "description": "str",
                                "examples": {
                                    "str": {"description": "str", "externalValue": "str", "summary": "str", "value": {}}
                                },
                                "required": bool,
                                "schemaId": "str",
                                "typeName": "str",
                                "values": ["str"],
                            }
                        ],
                        "representations": [
                            {
                                "contentType": "str",
                                "examples": {
                                    "str": {"description": "str", "externalValue": "str", "summary": "str", "value": {}}
                                },
                                "formParameters": [
                                    {
                                        "name": "str",
                                        "type": "str",
                                        "defaultValue": "str",
                                        "description": "str",
                                        "examples": {
                                            "str": {
                                                "description": "str",
                                                "externalValue": "str",
                                                "summary": "str",
                                                "value": {},
                                            }
                                        },
                                        "required": bool,
                                        "schemaId": "str",
                                        "typeName": "str",
                                        "values": ["str"],
                                    }
                                ],
                                "schemaId": "str",
                                "typeName": "str",
                            }
                        ],
                    },
                    "responses": [
                        {
                            "statusCode": 0,
                            "description": "str",
                            "headers": [
                                {
                                    "name": "str",
                                    "type": "str",
                                    "defaultValue": "str",
                                    "description": "str",
                                    "examples": {
                                        "str": {
                                            "description": "str",
                                            "externalValue": "str",
                                            "summary": "str",
                                            "value": {},
                                        }
                                    },
                                    "required": bool,
                                    "schemaId": "str",
                                    "typeName": "str",
                                    "values": ["str"],
                                }
                            ],
                            "representations": [
                                {
                                    "contentType": "str",
                                    "examples": {
                                        "str": {
                                            "description": "str",
                                            "externalValue": "str",
                                            "summary": "str",
                                            "value": {},
                                        }
                                    },
                                    "formParameters": [
                                        {
                                            "name": "str",
                                            "type": "str",
                                            "defaultValue": "str",
                                            "description": "str",
                                            "examples": {
                                                "str": {
                                                    "description": "str",
                                                    "externalValue": "str",
                                                    "summary": "str",
                                                    "value": {},
                                                }
                                            },
                                            "required": bool,
                                            "schemaId": "str",
                                            "typeName": "str",
                                            "values": ["str"],
                                        }
                                    ],
                                    "schemaId": "str",
                                    "typeName": "str",
                                }
                            ],
                        }
                    ],
                    "templateParameters": [
                        {
                            "name": "str",
                            "type": "str",
                            "defaultValue": "str",
                            "description": "str",
                            "examples": {
                                "str": {"description": "str", "externalValue": "str", "summary": "str", "value": {}}
                            },
                            "required": bool,
                            "schemaId": "str",
                            "typeName": "str",
                            "values": ["str"],
                        }
                    ],
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_api_operation_update(self, resource_group):
        response = await self.client.workspace_api_operation.update(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            api_id="str",
            operation_id="str",
            parameters={
                "properties": {
                    "description": "str",
                    "displayName": "str",
                    "method": "str",
                    "policies": "str",
                    "request": {
                        "description": "str",
                        "headers": [
                            {
                                "name": "str",
                                "type": "str",
                                "defaultValue": "str",
                                "description": "str",
                                "examples": {
                                    "str": {"description": "str", "externalValue": "str", "summary": "str", "value": {}}
                                },
                                "required": bool,
                                "schemaId": "str",
                                "typeName": "str",
                                "values": ["str"],
                            }
                        ],
                        "queryParameters": [
                            {
                                "name": "str",
                                "type": "str",
                                "defaultValue": "str",
                                "description": "str",
                                "examples": {
                                    "str": {"description": "str", "externalValue": "str", "summary": "str", "value": {}}
                                },
                                "required": bool,
                                "schemaId": "str",
                                "typeName": "str",
                                "values": ["str"],
                            }
                        ],
                        "representations": [
                            {
                                "contentType": "str",
                                "examples": {
                                    "str": {"description": "str", "externalValue": "str", "summary": "str", "value": {}}
                                },
                                "formParameters": [
                                    {
                                        "name": "str",
                                        "type": "str",
                                        "defaultValue": "str",
                                        "description": "str",
                                        "examples": {
                                            "str": {
                                                "description": "str",
                                                "externalValue": "str",
                                                "summary": "str",
                                                "value": {},
                                            }
                                        },
                                        "required": bool,
                                        "schemaId": "str",
                                        "typeName": "str",
                                        "values": ["str"],
                                    }
                                ],
                                "schemaId": "str",
                                "typeName": "str",
                            }
                        ],
                    },
                    "responses": [
                        {
                            "statusCode": 0,
                            "description": "str",
                            "headers": [
                                {
                                    "name": "str",
                                    "type": "str",
                                    "defaultValue": "str",
                                    "description": "str",
                                    "examples": {
                                        "str": {
                                            "description": "str",
                                            "externalValue": "str",
                                            "summary": "str",
                                            "value": {},
                                        }
                                    },
                                    "required": bool,
                                    "schemaId": "str",
                                    "typeName": "str",
                                    "values": ["str"],
                                }
                            ],
                            "representations": [
                                {
                                    "contentType": "str",
                                    "examples": {
                                        "str": {
                                            "description": "str",
                                            "externalValue": "str",
                                            "summary": "str",
                                            "value": {},
                                        }
                                    },
                                    "formParameters": [
                                        {
                                            "name": "str",
                                            "type": "str",
                                            "defaultValue": "str",
                                            "description": "str",
                                            "examples": {
                                                "str": {
                                                    "description": "str",
                                                    "externalValue": "str",
                                                    "summary": "str",
                                                    "value": {},
                                                }
                                            },
                                            "required": bool,
                                            "schemaId": "str",
                                            "typeName": "str",
                                            "values": ["str"],
                                        }
                                    ],
                                    "schemaId": "str",
                                    "typeName": "str",
                                }
                            ],
                        }
                    ],
                    "templateParameters": [
                        {
                            "name": "str",
                            "type": "str",
                            "defaultValue": "str",
                            "description": "str",
                            "examples": {
                                "str": {"description": "str", "externalValue": "str", "summary": "str", "value": {}}
                            },
                            "required": bool,
                            "schemaId": "str",
                            "typeName": "str",
                            "values": ["str"],
                        }
                    ],
                    "urlTemplate": "str",
                }
            },
            etag="str",
            match_condition=~azure.core.MatchConditions,
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_api_operation_delete(self, resource_group):
        response = await self.client.workspace_api_operation.delete(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            api_id="str",
            operation_id="str",
            etag="str",
            match_condition=~azure.core.MatchConditions,
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_api_operation_list_by_api(self, resource_group):
        response = self.client.workspace_api_operation.list_by_api(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            api_id="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...
