# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.compute import ComputeManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-compute
# USAGE
    python dedicated_host_update_minimum_set_gen.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ComputeManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.dedicated_hosts.begin_update(
        resource_group_name="rgcompute",
        host_group_name="aa",
        host_name="aaaaaaaaaaaaaaaaaaaaaaaaaa",
        parameters={
            "properties": {
                "autoReplaceOnFailure": bool,
                "hostId": "str",
                "instanceView": {
                    "assetId": "str",
                    "availableCapacity": {"allocatableVMs": [{"count": 0.0, "vmSize": "str"}]},
                    "statuses": [
                        {
                            "code": "str",
                            "displayStatus": "str",
                            "level": "str",
                            "message": "str",
                            "time": "2020-02-20 00:00:00",
                        }
                    ],
                },
                "licenseType": "str",
                "platformFaultDomain": 0,
                "provisioningState": "str",
                "provisioningTime": "2020-02-20 00:00:00",
                "timeCreated": "2020-02-20 00:00:00",
                "virtualMachines": [{"id": "str"}],
            },
            "sku": {"capacity": 0, "name": "str", "tier": "str"},
            "tags": {"str": "str"},
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-11-01/dedicatedHostExamples/DedicatedHost_Update_MinimumSet_Gen.json
if __name__ == "__main__":
    main()
