# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Optional, TYPE_CHECKING, cast
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.settings import settings
from azure.mgmt.core import ARMPipelineClient
from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy
from azure.mgmt.core.tools import get_arm_endpoints

from ._configuration import DataProtectionMgmtClientConfiguration
from ._utils.serialization import Deserializer, Serializer
from .operations import (
    BackupInstancesExtensionRoutingOperations,
    BackupInstancesOperations,
    BackupPoliciesOperations,
    BackupVaultOperationResultsOperations,
    BackupVaultsOperations,
    DataProtectionOperations,
    DataProtectionOperationsOperations,
    DeletedBackupInstancesOperations,
    DeletedBackupVaultsOperations,
    DppResourceGuardProxyOperations,
    ExportJobsOperationResultOperations,
    ExportJobsOperations,
    FetchCrossRegionRestoreJobOperations,
    FetchCrossRegionRestoreJobsOperations,
    FetchSecondaryRecoveryPointsOperations,
    JobsOperations,
    OperationResultOperations,
    OperationStatusBackupVaultContextOperations,
    OperationStatusOperations,
    OperationStatusResourceGroupContextOperations,
    RecoveryPointsOperations,
    ResourceGuardsOperations,
    RestorableTimeRangesOperations,
)

if TYPE_CHECKING:
    from azure.core import AzureClouds
    from azure.core.credentials import TokenCredential


class DataProtectionMgmtClient:  # pylint: disable=too-many-instance-attributes
    """Open API 2.0 Specs for Azure Data Protection service.

    :ivar data_protection_operations: DataProtectionOperationsOperations operations
    :vartype data_protection_operations:
     azure.mgmt.dataprotection.operations.DataProtectionOperationsOperations
    :ivar backup_instances: BackupInstancesOperations operations
    :vartype backup_instances: azure.mgmt.dataprotection.operations.BackupInstancesOperations
    :ivar backup_vault_operation_results: BackupVaultOperationResultsOperations operations
    :vartype backup_vault_operation_results:
     azure.mgmt.dataprotection.operations.BackupVaultOperationResultsOperations
    :ivar deleted_backup_vaults: DeletedBackupVaultsOperations operations
    :vartype deleted_backup_vaults:
     azure.mgmt.dataprotection.operations.DeletedBackupVaultsOperations
    :ivar resource_guards: ResourceGuardsOperations operations
    :vartype resource_guards: azure.mgmt.dataprotection.operations.ResourceGuardsOperations
    :ivar backup_vaults: BackupVaultsOperations operations
    :vartype backup_vaults: azure.mgmt.dataprotection.operations.BackupVaultsOperations
    :ivar operation_status_backup_vault_context: OperationStatusBackupVaultContextOperations
     operations
    :vartype operation_status_backup_vault_context:
     azure.mgmt.dataprotection.operations.OperationStatusBackupVaultContextOperations
    :ivar export_jobs: ExportJobsOperations operations
    :vartype export_jobs: azure.mgmt.dataprotection.operations.ExportJobsOperations
    :ivar export_jobs_operation_result: ExportJobsOperationResultOperations operations
    :vartype export_jobs_operation_result:
     azure.mgmt.dataprotection.operations.ExportJobsOperationResultOperations
    :ivar backup_policies: BackupPoliciesOperations operations
    :vartype backup_policies: azure.mgmt.dataprotection.operations.BackupPoliciesOperations
    :ivar restorable_time_ranges: RestorableTimeRangesOperations operations
    :vartype restorable_time_ranges:
     azure.mgmt.dataprotection.operations.RestorableTimeRangesOperations
    :ivar recovery_points: RecoveryPointsOperations operations
    :vartype recovery_points: azure.mgmt.dataprotection.operations.RecoveryPointsOperations
    :ivar jobs: JobsOperations operations
    :vartype jobs: azure.mgmt.dataprotection.operations.JobsOperations
    :ivar deleted_backup_instances: DeletedBackupInstancesOperations operations
    :vartype deleted_backup_instances:
     azure.mgmt.dataprotection.operations.DeletedBackupInstancesOperations
    :ivar dpp_resource_guard_proxy: DppResourceGuardProxyOperations operations
    :vartype dpp_resource_guard_proxy:
     azure.mgmt.dataprotection.operations.DppResourceGuardProxyOperations
    :ivar operation_result: OperationResultOperations operations
    :vartype operation_result: azure.mgmt.dataprotection.operations.OperationResultOperations
    :ivar operation_status: OperationStatusOperations operations
    :vartype operation_status: azure.mgmt.dataprotection.operations.OperationStatusOperations
    :ivar operation_status_resource_group_context: OperationStatusResourceGroupContextOperations
     operations
    :vartype operation_status_resource_group_context:
     azure.mgmt.dataprotection.operations.OperationStatusResourceGroupContextOperations
    :ivar data_protection: DataProtectionOperations operations
    :vartype data_protection: azure.mgmt.dataprotection.operations.DataProtectionOperations
    :ivar fetch_secondary_recovery_points: FetchSecondaryRecoveryPointsOperations operations
    :vartype fetch_secondary_recovery_points:
     azure.mgmt.dataprotection.operations.FetchSecondaryRecoveryPointsOperations
    :ivar fetch_cross_region_restore_job: FetchCrossRegionRestoreJobOperations operations
    :vartype fetch_cross_region_restore_job:
     azure.mgmt.dataprotection.operations.FetchCrossRegionRestoreJobOperations
    :ivar fetch_cross_region_restore_jobs: FetchCrossRegionRestoreJobsOperations operations
    :vartype fetch_cross_region_restore_jobs:
     azure.mgmt.dataprotection.operations.FetchCrossRegionRestoreJobsOperations
    :ivar backup_instances_extension_routing: BackupInstancesExtensionRoutingOperations operations
    :vartype backup_instances_extension_routing:
     azure.mgmt.dataprotection.operations.BackupInstancesExtensionRoutingOperations
    :param credential: Credential used to authenticate requests to the service. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
    :type subscription_id: str
    :param base_url: Service host. Default value is None.
    :type base_url: str
    :keyword cloud_setting: The cloud setting for which to get the ARM endpoint. Default value is
     None.
    :paramtype cloud_setting: ~azure.core.AzureClouds
    :keyword api_version: The API version to use for this operation. Known values are "2026-03-01".
     Default value is "2026-03-01". Note that overriding this default value may result in
     unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "TokenCredential",
        subscription_id: str,
        base_url: Optional[str] = None,
        *,
        cloud_setting: Optional["AzureClouds"] = None,
        **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        _cloud = cloud_setting or settings.current.azure_cloud  # type: ignore
        _endpoints = get_arm_endpoints(_cloud)
        if not base_url:
            base_url = _endpoints["resource_manager"]
        credential_scopes = kwargs.pop("credential_scopes", _endpoints["credential_scopes"])
        self._config = DataProtectionMgmtClientConfiguration(
            credential=credential,
            subscription_id=subscription_id,
            base_url=cast(str, base_url),
            cloud_setting=cloud_setting,
            credential_scopes=credential_scopes,
            **kwargs
        )

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                ARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=cast(str, _endpoint), policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.data_protection_operations = DataProtectionOperationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_instances = BackupInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_vault_operation_results = BackupVaultOperationResultsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.deleted_backup_vaults = DeletedBackupVaultsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.resource_guards = ResourceGuardsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.backup_vaults = BackupVaultsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.operation_status_backup_vault_context = OperationStatusBackupVaultContextOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.export_jobs = ExportJobsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.export_jobs_operation_result = ExportJobsOperationResultOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_policies = BackupPoliciesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.restorable_time_ranges = RestorableTimeRangesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.recovery_points = RecoveryPointsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.jobs = JobsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.deleted_backup_instances = DeletedBackupInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.dpp_resource_guard_proxy = DppResourceGuardProxyOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.operation_result = OperationResultOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.operation_status = OperationStatusOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.operation_status_resource_group_context = OperationStatusResourceGroupContextOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.data_protection = DataProtectionOperations(self._client, self._config, self._serialize, self._deserialize)
        self.fetch_secondary_recovery_points = FetchSecondaryRecoveryPointsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.fetch_cross_region_restore_job = FetchCrossRegionRestoreJobOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.fetch_cross_region_restore_jobs = FetchCrossRegionRestoreJobsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_instances_extension_routing = BackupInstancesExtensionRoutingOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client.send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)
