# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    AssignDeploymentResourcesDetails,
    AssignedDeploymentResource,
    AssignedProjectDeploymentMetadata,
    AssignedProjectDeploymentsMetadata,
    ConfusionMatrixCell,
    ConfusionMatrixRow,
    CopyProjectDetails,
    CopyProjectState,
    CreateDeploymentDetails,
    CreateProjectOptions,
    CustomEntityRecognitionDocumentEvalResult,
    CustomEntityRecognitionEvalSummary,
    CustomHealthcareDocumentEvalResult,
    CustomHealthcareEvalSummary,
    CustomMultiLabelClassificationDocumentEvalResult,
    CustomMultiLabelClassificationEvalSummary,
    CustomSingleLabelClassificationDocumentEvalResult,
    CustomSingleLabelClassificationEvalSummary,
    CustomTextSentimentDocumentEvalResult,
    CustomTextSentimentEvalSummary,
    CustomTextSentimentProjectAssets,
    DataGenerationConnectionInfo,
    DataGenerationSetting,
    DeleteDeploymentDetails,
    DeploymentDeleteFromResourcesState,
    DeploymentResource,
    DeploymentResourcesState,
    DeploymentState,
    DocumentEntityLabelEvalResult,
    DocumentEntityRecognitionEvalResult,
    DocumentEntityRegionEvalResult,
    DocumentEvalResult,
    DocumentHealthcareEvalResult,
    DocumentMultiLabelClassificationEvalResult,
    DocumentSentimentLabelEvalResult,
    DocumentSingleLabelClassificationEvalResult,
    DocumentTextSentimentEvalResult,
    EntityEvalSummary,
    EntityRecognitionEvalSummary,
    EvalSummary,
    EvaluationDetails,
    EvaluationJobResult,
    EvaluationState,
    ExportProjectState,
    ExportedClass,
    ExportedCompositeEntity,
    ExportedCustomAbstractiveSummarizationDocument,
    ExportedCustomAbstractiveSummarizationProjectAsset,
    ExportedCustomEntityRecognitionDocument,
    ExportedCustomEntityRecognitionProjectAsset,
    ExportedCustomHealthcareDocument,
    ExportedCustomHealthcareProjectAsset,
    ExportedCustomMultiLabelClassificationDocument,
    ExportedCustomMultiLabelClassificationProjectAsset,
    ExportedCustomSingleLabelClassificationDocument,
    ExportedCustomSingleLabelClassificationProjectAsset,
    ExportedCustomTextSentimentDocument,
    ExportedDocumentClass,
    ExportedDocumentEntityLabel,
    ExportedDocumentEntityRegion,
    ExportedDocumentSentimentLabel,
    ExportedEntity,
    ExportedEntityList,
    ExportedEntityListSynonym,
    ExportedEntitySublist,
    ExportedModelDetails,
    ExportedModelManifest,
    ExportedModelState,
    ExportedPrebuiltEntity,
    ExportedProject,
    ExportedProjectAsset,
    ExportedTrainedModel,
    ImportProjectState,
    LoadSnapshotState,
    ModelFile,
    MultiLabelClassEvalSummary,
    MultiLabelClassificationEvalSummary,
    PrebuiltEntity,
    ProjectDeletionState,
    ProjectDeployment,
    ProjectDetails,
    ProjectSettings,
    ProjectTrainedModel,
    ResourceMetadata,
    SentimentEvalSummary,
    SingleLabelClassEvalSummary,
    SingleLabelClassificationEvalSummary,
    SpanSentimentEvalSummary,
    SubTrainingState,
    SupportedLanguage,
    SwapDeploymentsDetails,
    SwapDeploymentsState,
    TextSentimentEvalSummary,
    TrainingConfigVersion,
    TrainingJobDetails,
    TrainingJobResult,
    TrainingState,
    UnassignDeploymentResourcesDetails,
)

from ._enums import (  # type: ignore
    CompositionMode,
    EvaluationKind,
    OperationStatus,
    ProjectKind,
    Sentiment,
    StringIndexType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AssignDeploymentResourcesDetails",
    "AssignedDeploymentResource",
    "AssignedProjectDeploymentMetadata",
    "AssignedProjectDeploymentsMetadata",
    "ConfusionMatrixCell",
    "ConfusionMatrixRow",
    "CopyProjectDetails",
    "CopyProjectState",
    "CreateDeploymentDetails",
    "CreateProjectOptions",
    "CustomEntityRecognitionDocumentEvalResult",
    "CustomEntityRecognitionEvalSummary",
    "CustomHealthcareDocumentEvalResult",
    "CustomHealthcareEvalSummary",
    "CustomMultiLabelClassificationDocumentEvalResult",
    "CustomMultiLabelClassificationEvalSummary",
    "CustomSingleLabelClassificationDocumentEvalResult",
    "CustomSingleLabelClassificationEvalSummary",
    "CustomTextSentimentDocumentEvalResult",
    "CustomTextSentimentEvalSummary",
    "CustomTextSentimentProjectAssets",
    "DataGenerationConnectionInfo",
    "DataGenerationSetting",
    "DeleteDeploymentDetails",
    "DeploymentDeleteFromResourcesState",
    "DeploymentResource",
    "DeploymentResourcesState",
    "DeploymentState",
    "DocumentEntityLabelEvalResult",
    "DocumentEntityRecognitionEvalResult",
    "DocumentEntityRegionEvalResult",
    "DocumentEvalResult",
    "DocumentHealthcareEvalResult",
    "DocumentMultiLabelClassificationEvalResult",
    "DocumentSentimentLabelEvalResult",
    "DocumentSingleLabelClassificationEvalResult",
    "DocumentTextSentimentEvalResult",
    "EntityEvalSummary",
    "EntityRecognitionEvalSummary",
    "EvalSummary",
    "EvaluationDetails",
    "EvaluationJobResult",
    "EvaluationState",
    "ExportProjectState",
    "ExportedClass",
    "ExportedCompositeEntity",
    "ExportedCustomAbstractiveSummarizationDocument",
    "ExportedCustomAbstractiveSummarizationProjectAsset",
    "ExportedCustomEntityRecognitionDocument",
    "ExportedCustomEntityRecognitionProjectAsset",
    "ExportedCustomHealthcareDocument",
    "ExportedCustomHealthcareProjectAsset",
    "ExportedCustomMultiLabelClassificationDocument",
    "ExportedCustomMultiLabelClassificationProjectAsset",
    "ExportedCustomSingleLabelClassificationDocument",
    "ExportedCustomSingleLabelClassificationProjectAsset",
    "ExportedCustomTextSentimentDocument",
    "ExportedDocumentClass",
    "ExportedDocumentEntityLabel",
    "ExportedDocumentEntityRegion",
    "ExportedDocumentSentimentLabel",
    "ExportedEntity",
    "ExportedEntityList",
    "ExportedEntityListSynonym",
    "ExportedEntitySublist",
    "ExportedModelDetails",
    "ExportedModelManifest",
    "ExportedModelState",
    "ExportedPrebuiltEntity",
    "ExportedProject",
    "ExportedProjectAsset",
    "ExportedTrainedModel",
    "ImportProjectState",
    "LoadSnapshotState",
    "ModelFile",
    "MultiLabelClassEvalSummary",
    "MultiLabelClassificationEvalSummary",
    "PrebuiltEntity",
    "ProjectDeletionState",
    "ProjectDeployment",
    "ProjectDetails",
    "ProjectSettings",
    "ProjectTrainedModel",
    "ResourceMetadata",
    "SentimentEvalSummary",
    "SingleLabelClassEvalSummary",
    "SingleLabelClassificationEvalSummary",
    "SpanSentimentEvalSummary",
    "SubTrainingState",
    "SupportedLanguage",
    "SwapDeploymentsDetails",
    "SwapDeploymentsState",
    "TextSentimentEvalSummary",
    "TrainingConfigVersion",
    "TrainingJobDetails",
    "TrainingJobResult",
    "TrainingState",
    "UnassignDeploymentResourcesDetails",
    "CompositionMode",
    "EvaluationKind",
    "OperationStatus",
    "ProjectKind",
    "Sentiment",
    "StringIndexType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
