# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.billingbenefits import BillingBenefitsMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-billingbenefits
# USAGE
    python macc_validate_create_milestones_on_existing_macc.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BillingBenefitsMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.benefit.validate(
        body={
            "benefits": [
                {
                    "benefitType": "MACC",
                    "properties": {
                        "billingAccountResourceId": "/providers/Microsoft.Billing/billingAccounts/{acctId:orgId}",
                        "commitment": {"amount": 20000, "currencyCode": "USD", "grain": "FullTerm"},
                        "endAt": "2026-07-31T23:59:59Z",
                        "entityType": "Primary",
                        "milestones": [
                            {"commitment": {"amount": 7000, "currencyCode": "USD"}, "endAt": "2024-10-31T23:59:59Z"},
                            {"commitment": {"amount": 13000, "currencyCode": "USD"}, "endAt": "2025-01-31T23:59:59Z"},
                        ],
                        "resourceId": "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/myResourceGroup/providers/Microsoft.BillingBenefits/benefits/myBenefit",
                        "systemId": "13810867107109237",
                    },
                }
            ]
        },
    )
    print(response)


# x-ms-original-file: 2025-12-01-preview/MaccValidateCreateMilestonesOnExistingMacc.json
if __name__ == "__main__":
    main()
