# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.containerregistry import ContainerRegistryManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-containerregistry
# USAGE
    python pipeline_run_create_export.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ContainerRegistryManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.pipeline_runs.begin_create(
        resource_group_name="myResourceGroup",
        registry_name="myRegistry",
        pipeline_run_name="myPipelineRun",
        pipeline_run_create_parameters={
            "properties": {
                "request": {
                    "artifacts": [
                        "sourceRepository/hello-world",
                        "sourceRepository2@sha256:00000000000000000000000000000000000",
                    ],
                    "pipelineResourceId": "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/myResourceGroup/providers/Microsoft.ContainerRegistry/registries/myRegistry/exportPipelines/myExportPipeline",
                    "target": {"name": "myblob.tar.gz", "type": "AzureStorageBlob"},
                }
            }
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-01-01-preview/PipelineRunCreate_Export.json
if __name__ == "__main__":
    main()
