# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.azurestackhci import AzureStackHCIClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAzureStackHCIDeploymentSettingsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureStackHCIClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_deployment_settings_get(self, resource_group):
        response = self.client.deployment_settings.get(
            resource_group_name=resource_group.name,
            cluster_name="str",
            deployment_settings_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_deployment_settings_begin_create_or_update(self, resource_group):
        response = self.client.deployment_settings.begin_create_or_update(
            resource_group_name=resource_group.name,
            cluster_name="str",
            resource={
                "id": "str",
                "name": "str",
                "properties": {
                    "arcNodeResourceIds": ["str"],
                    "deploymentConfiguration": {
                        "scaleUnits": [
                            {
                                "deploymentData": {
                                    "adouPath": "str",
                                    "assemblyInfo": {
                                        "packageVersion": "str",
                                        "payload": [
                                            {"fileName": "str", "hash": "str", "identifier": "str", "url": "str"}
                                        ],
                                    },
                                    "cluster": {
                                        "azureServiceEndpoint": "str",
                                        "cloudAccountName": "str",
                                        "clusterPattern": "str",
                                        "hardwareClass": "str",
                                        "name": "str",
                                        "witnessPath": "str",
                                        "witnessType": "str",
                                    },
                                    "domainFqdn": "str",
                                    "hostNetwork": {
                                        "enableStorageAutoIp": bool,
                                        "intents": [
                                            {
                                                "adapter": ["str"],
                                                "adapterPropertyOverrides": {
                                                    "jumboPacket": "str",
                                                    "networkDirect": "str",
                                                    "networkDirectTechnology": "str",
                                                },
                                                "name": "str",
                                                "overrideAdapterProperty": bool,
                                                "overrideQosPolicy": bool,
                                                "overrideVirtualSwitchConfiguration": bool,
                                                "qosPolicyOverrides": {
                                                    "bandwidthPercentage_SMB": "str",
                                                    "priorityValue8021Action_Cluster": "str",
                                                    "priorityValue8021Action_SMB": "str",
                                                },
                                                "trafficType": ["str"],
                                                "virtualSwitchConfigurationOverrides": {
                                                    "enableIov": "str",
                                                    "loadBalancingAlgorithm": "str",
                                                },
                                            }
                                        ],
                                        "sanNetworks": {
                                            "clusterNetworkConfig": {
                                                "adapterIPConfig": [
                                                    {
                                                        "addressPrefix": "str",
                                                        "name": "str",
                                                        "networkAdapterName": "str",
                                                        "vlanId": 0,
                                                    }
                                                ],
                                                "adapterProperties": {
                                                    "bandwidthPercentageSmb": 0,
                                                    "jumboPacket": 0,
                                                    "priorityValue8021ActionCluster": 0,
                                                    "priorityValue8021ActionSmb": 0,
                                                },
                                            }
                                        },
                                        "storageConnectivitySwitchless": bool,
                                        "storageNetworks": [
                                            {
                                                "name": "str",
                                                "networkAdapterName": "str",
                                                "storageAdapterIPInfo": [
                                                    {"ipv4Address": "str", "physicalNode": "str", "subnetMask": "str"}
                                                ],
                                                "vlanId": "str",
                                            }
                                        ],
                                    },
                                    "identityProvider": "str",
                                    "infrastructureNetwork": [
                                        {
                                            "dnsServerConfig": "str",
                                            "dnsServers": ["str"],
                                            "dnsZones": [{"dnsForwarder": ["str"], "dnsZoneName": "str"}],
                                            "gateway": "str",
                                            "ipPools": [{"endingAddress": "str", "startingAddress": "str"}],
                                            "subnetMask": "str",
                                            "useDhcp": bool,
                                        }
                                    ],
                                    "isManagementCluster": bool,
                                    "localAvailabilityZones": [{"localAvailabilityZoneName": "str", "nodes": ["str"]}],
                                    "namingPrefix": "str",
                                    "observability": {
                                        "episodicDataUpload": bool,
                                        "euLocation": bool,
                                        "streamingDataClient": bool,
                                    },
                                    "optionalServices": {"customLocation": "str"},
                                    "physicalNodes": [{"ipv4Address": "str", "name": "str"}],
                                    "sdnIntegration": {
                                        "networkController": {
                                            "macAddressPoolStart": "str",
                                            "macAddressPoolStop": "str",
                                            "networkVirtualizationEnabled": bool,
                                        }
                                    },
                                    "secrets": [{"eceSecretName": "str", "secretLocation": "str", "secretName": "str"}],
                                    "secretsLocation": "str",
                                    "securitySettings": {
                                        "bitlockerBootVolume": bool,
                                        "bitlockerDataVolumes": bool,
                                        "credentialGuardEnforced": bool,
                                        "driftControlEnforced": bool,
                                        "drtmProtection": bool,
                                        "hvciProtection": bool,
                                        "sideChannelMitigationEnforced": bool,
                                        "smbClusterEncryption": bool,
                                        "smbSigningEnforced": bool,
                                        "wdacEnforced": bool,
                                    },
                                    "storage": {
                                        "configurationMode": "str",
                                        "s2d": {"overprovisioningRatio": "str", "volumeType": "str"},
                                        "san": {"infraPerfLunId": "str", "infraVolLunId": "str"},
                                        "storageType": "str",
                                    },
                                },
                                "sbePartnerInfo": {
                                    "credentialList": [
                                        {"eceSecretName": "str", "secretLocation": "str", "secretName": "str"}
                                    ],
                                    "partnerProperties": [{"name": "str", "value": "str"}],
                                    "sbeDeploymentInfo": {
                                        "family": "str",
                                        "publisher": "str",
                                        "sbeManifestCreationDate": "2020-02-20 00:00:00",
                                        "sbeManifestSource": "str",
                                        "version": "str",
                                    },
                                },
                            }
                        ],
                        "version": "str",
                    },
                    "deploymentMode": "str",
                    "operationType": "str",
                    "provisioningState": "str",
                    "reportedProperties": {
                        "deploymentStatus": {
                            "status": "str",
                            "steps": [
                                {
                                    "description": "str",
                                    "endTimeUtc": "str",
                                    "exception": ["str"],
                                    "fullStepIndex": "str",
                                    "name": "str",
                                    "startTimeUtc": "str",
                                    "status": "str",
                                    "steps": [...],
                                }
                            ],
                        },
                        "validationStatus": {
                            "status": "str",
                            "steps": [
                                {
                                    "description": "str",
                                    "endTimeUtc": "str",
                                    "exception": ["str"],
                                    "fullStepIndex": "str",
                                    "name": "str",
                                    "startTimeUtc": "str",
                                    "status": "str",
                                    "steps": [...],
                                }
                            ],
                        },
                    },
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
            deployment_settings_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_deployment_settings_begin_delete(self, resource_group):
        response = self.client.deployment_settings.begin_delete(
            resource_group_name=resource_group.name,
            cluster_name="str",
            deployment_settings_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_deployment_settings_list_by_clusters(self, resource_group):
        response = self.client.deployment_settings.list_by_clusters(
            resource_group_name=resource_group.name,
            cluster_name="str",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...
