# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.deviceregistry import DeviceRegistryMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-deviceregistry
# USAGE
    python update_namespace_discovered_device.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DeviceRegistryMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.namespace_discovered_devices.begin_update(
        resource_group_name="myResourceGroup",
        namespace_name="my-namespace-1",
        discovered_device_name="my-discovereddevice-1",
        properties={
            "properties": {
                "endpoints": {
                    "outbound": {
                        "assigned": {
                            "newIothubEndpoint": {
                                "address": "https://iothub-for-dps.azure-devices.net",
                                "endpointType": "Microsoft.Devices/IotHubs",
                            }
                        }
                    }
                }
            }
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-03-01-preview/Update_NamespaceDiscoveredDevice.json
if __name__ == "__main__":
    main()
