# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.authorization.aio import AuthorizationManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAuthorizationManagementRoleDefinitionsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AuthorizationManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_role_definitions_get(self, resource_group):
        response = await self.client.role_definitions.get(
            scope="str",
            role_definition_id="str",
            api_version="2022-05-01-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_role_definitions_create_or_update(self, resource_group):
        response = await self.client.role_definitions.create_or_update(
            scope="str",
            role_definition_id="str",
            role_definition={
                "id": "str",
                "name": "str",
                "properties": {
                    "assignableScopes": ["str"],
                    "createdBy": "str",
                    "createdOn": "2020-02-20 00:00:00",
                    "description": "str",
                    "permissions": [
                        {
                            "actions": ["str"],
                            "condition": "str",
                            "conditionVersion": "str",
                            "dataActions": ["str"],
                            "notActions": ["str"],
                            "notDataActions": ["str"],
                        }
                    ],
                    "roleName": "str",
                    "type": "str",
                    "updatedBy": "str",
                    "updatedOn": "2020-02-20 00:00:00",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
            api_version="2022-05-01-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_role_definitions_delete(self, resource_group):
        response = await self.client.role_definitions.delete(
            scope="str",
            role_definition_id="str",
            api_version="2022-05-01-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_role_definitions_list(self, resource_group):
        response = self.client.role_definitions.list(
            scope="str",
            api_version="2022-05-01-preview",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_role_definitions_get_by_id(self, resource_group):
        response = await self.client.role_definitions.get_by_id(
            role_id="str",
            api_version="2022-05-01-preview",
        )

        # please add some check logic here by yourself
        # ...
