# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.compute.aio import ComputeManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestComputeManagementVirtualMachineRunCommandsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ComputeManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_run_commands_get_by_virtual_machine(self, resource_group):
        response = await self.client.virtual_machine_run_commands.get_by_virtual_machine(
            resource_group_name=resource_group.name,
            vm_name="str",
            run_command_name="str",
            api_version="2025-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_run_commands_begin_create_or_update(self, resource_group):
        response = await (
            await self.client.virtual_machine_run_commands.begin_create_or_update(
                resource_group_name=resource_group.name,
                vm_name="str",
                run_command_name="str",
                run_command={
                    "location": "str",
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "asyncExecution": bool,
                        "errorBlobManagedIdentity": {"clientId": "str", "objectId": "str"},
                        "errorBlobUri": "str",
                        "instanceView": {
                            "endTime": "2020-02-20 00:00:00",
                            "error": "str",
                            "executionMessage": "str",
                            "executionState": "str",
                            "exitCode": 0,
                            "output": "str",
                            "startTime": "2020-02-20 00:00:00",
                            "statuses": [
                                {
                                    "code": "str",
                                    "displayStatus": "str",
                                    "level": "str",
                                    "message": "str",
                                    "time": "2020-02-20 00:00:00",
                                }
                            ],
                        },
                        "outputBlobManagedIdentity": {"clientId": "str", "objectId": "str"},
                        "outputBlobUri": "str",
                        "parameters": [{"name": "str", "value": "str"}],
                        "protectedParameters": [{"name": "str", "value": "str"}],
                        "provisioningState": "str",
                        "runAsPassword": "str",
                        "runAsUser": "str",
                        "source": {
                            "commandId": "str",
                            "galleryScriptReferenceId": "str",
                            "script": "str",
                            "scriptShell": "str",
                            "scriptUri": "str",
                            "scriptUriManagedIdentity": {"clientId": "str", "objectId": "str"},
                        },
                        "timeoutInSeconds": 0,
                        "treatFailureAsDeploymentFailure": bool,
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
                api_version="2025-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_run_commands_begin_update(self, resource_group):
        response = await (
            await self.client.virtual_machine_run_commands.begin_update(
                resource_group_name=resource_group.name,
                vm_name="str",
                run_command_name="str",
                run_command={
                    "properties": {
                        "asyncExecution": bool,
                        "errorBlobManagedIdentity": {"clientId": "str", "objectId": "str"},
                        "errorBlobUri": "str",
                        "instanceView": {
                            "endTime": "2020-02-20 00:00:00",
                            "error": "str",
                            "executionMessage": "str",
                            "executionState": "str",
                            "exitCode": 0,
                            "output": "str",
                            "startTime": "2020-02-20 00:00:00",
                            "statuses": [
                                {
                                    "code": "str",
                                    "displayStatus": "str",
                                    "level": "str",
                                    "message": "str",
                                    "time": "2020-02-20 00:00:00",
                                }
                            ],
                        },
                        "outputBlobManagedIdentity": {"clientId": "str", "objectId": "str"},
                        "outputBlobUri": "str",
                        "parameters": [{"name": "str", "value": "str"}],
                        "protectedParameters": [{"name": "str", "value": "str"}],
                        "provisioningState": "str",
                        "runAsPassword": "str",
                        "runAsUser": "str",
                        "source": {
                            "commandId": "str",
                            "galleryScriptReferenceId": "str",
                            "script": "str",
                            "scriptShell": "str",
                            "scriptUri": "str",
                            "scriptUriManagedIdentity": {"clientId": "str", "objectId": "str"},
                        },
                        "timeoutInSeconds": 0,
                        "treatFailureAsDeploymentFailure": bool,
                    },
                    "tags": {"str": "str"},
                },
                api_version="2025-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_run_commands_begin_delete(self, resource_group):
        response = await (
            await self.client.virtual_machine_run_commands.begin_delete(
                resource_group_name=resource_group.name,
                vm_name="str",
                run_command_name="str",
                api_version="2025-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_run_commands_list_by_virtual_machine(self, resource_group):
        response = self.client.virtual_machine_run_commands.list_by_virtual_machine(
            resource_group_name=resource_group.name,
            vm_name="str",
            api_version="2025-11-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_run_commands_list(self, resource_group):
        response = self.client.virtual_machine_run_commands.list(
            location="str",
            api_version="2025-11-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_run_commands_get(self, resource_group):
        response = await self.client.virtual_machine_run_commands.get(
            location="str",
            command_id="str",
            api_version="2025-11-01",
        )

        # please add some check logic here by yourself
        # ...
