# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    AFDDomain,
    AFDDomainHttpsCustomizedCipherSuiteSet,
    AFDDomainHttpsParameters,
    AFDDomainProperties,
    AFDDomainUpdateParameters,
    AFDDomainUpdatePropertiesParameters,
    AFDEndpoint,
    AFDEndpointProperties,
    AFDEndpointPropertiesUpdateParameters,
    AFDEndpointUpdateParameters,
    AFDOrigin,
    AFDOriginGroup,
    AFDOriginGroupProperties,
    AFDOriginGroupUpdateParameters,
    AFDOriginGroupUpdatePropertiesParameters,
    AFDOriginProperties,
    AFDOriginUpdateParameters,
    AFDOriginUpdatePropertiesParameters,
    AFDStateProperties,
    ActivatedResourceReference,
    AfdPurgeParameters,
    AfdRouteCacheConfiguration,
    AzureFirstPartyManagedCertificateParameters,
    BatchRuleProperties,
    CacheConfiguration,
    CacheExpirationActionParameters,
    CacheKeyQueryStringActionParameters,
    CanMigrateParameters,
    CanMigrateProperties,
    CanMigrateResult,
    CdnCertificateSourceParameters,
    CdnEndpoint,
    CdnManagedHttpsParameters,
    CdnMigrationToAfdParameters,
    CdnWebApplicationFirewallPolicy,
    CdnWebApplicationFirewallPolicyPatchParameters,
    CdnWebApplicationFirewallPolicyProperties,
    CertificateSourceParameters,
    CheckEndpointNameAvailabilityInput,
    CheckEndpointNameAvailabilityOutput,
    CheckHostNameAvailabilityInput,
    CheckNameAvailabilityInput,
    CheckNameAvailabilityOutput,
    CidrIpAddress,
    ClientPortMatchConditionParameters,
    Components18OrqelSchemasWafmetricsresponsePropertiesSeriesItemsPropertiesDataItems,
    Components1Gs0LlpSchemasMetricsresponsePropertiesSeriesItemsPropertiesDataItems,
    ComponentsKpo1PjSchemasWafrankingsresponsePropertiesDataItemsPropertiesMetricsItems,
    CompressionSettings,
    ContinentsResponse,
    ContinentsResponseContinentsItem,
    ContinentsResponseCountryOrRegionsItem,
    CookiesMatchConditionParameters,
    CustomDomain,
    CustomDomainHttpsParameters,
    CustomDomainParameters,
    CustomDomainProperties,
    CustomDomainPropertiesParameters,
    CustomRule,
    CustomRuleList,
    CustomerCertificateParameters,
    DeepCreatedCustomDomain,
    DeepCreatedCustomDomainProperties,
    DeepCreatedOrigin,
    DeepCreatedOriginGroup,
    DeepCreatedOriginGroupProperties,
    DeepCreatedOriginProperties,
    DeliveryRule,
    DeliveryRuleAction,
    DeliveryRuleActionParameters,
    DeliveryRuleCacheExpirationAction,
    DeliveryRuleCacheKeyQueryStringAction,
    DeliveryRuleClientPortCondition,
    DeliveryRuleCondition,
    DeliveryRuleConditionParameters,
    DeliveryRuleCookiesCondition,
    DeliveryRuleHostNameCondition,
    DeliveryRuleHttpVersionCondition,
    DeliveryRuleIsDeviceCondition,
    DeliveryRulePostArgsCondition,
    DeliveryRuleQueryStringCondition,
    DeliveryRuleRemoteAddressCondition,
    DeliveryRuleRequestBodyCondition,
    DeliveryRuleRequestHeaderAction,
    DeliveryRuleRequestHeaderCondition,
    DeliveryRuleRequestMethodCondition,
    DeliveryRuleRequestSchemeCondition,
    DeliveryRuleRequestUriCondition,
    DeliveryRuleResponseHeaderAction,
    DeliveryRuleRouteConfigurationOverrideAction,
    DeliveryRuleServerPortCondition,
    DeliveryRuleSocketAddrCondition,
    DeliveryRuleSslProtocolCondition,
    DeliveryRuleUrlFileExtensionCondition,
    DeliveryRuleUrlFileNameCondition,
    DeliveryRuleUrlPathCondition,
    DimensionProperties,
    DomainValidationProperties,
    EdgeNode,
    EdgeNodeProperties,
    Endpoint,
    EndpointProperties,
    EndpointPropertiesUpdateParameters,
    EndpointPropertiesUpdateParametersDeliveryPolicy,
    EndpointPropertiesUpdateParametersWebApplicationFirewallPolicyLink,
    EndpointUpdateParameters,
    ErrorAdditionalInfo,
    ErrorDetail,
    ErrorResponse,
    GeoFilter,
    HeaderActionParameters,
    HealthProbeParameters,
    HostNameMatchConditionParameters,
    HttpErrorRangeParameters,
    HttpVersionMatchConditionParameters,
    IpAddressGroup,
    IsDeviceMatchConditionParameters,
    KeyVaultCertificateSourceParameters,
    KeyVaultSigningKeyParameters,
    LoadBalancingSettingsParameters,
    LoadParameters,
    LogSpecification,
    ManagedCertificateParameters,
    ManagedRuleDefinition,
    ManagedRuleGroupDefinition,
    ManagedRuleGroupOverride,
    ManagedRuleOverride,
    ManagedRuleSet,
    ManagedRuleSetDefinition,
    ManagedRuleSetDefinitionProperties,
    ManagedRuleSetList,
    ManagedServiceIdentity,
    MatchCondition,
    MetricAvailability,
    MetricSpecification,
    MetricsResponse,
    MetricsResponseSeriesItem,
    MetricsResponseSeriesPropertiesItemsItem,
    MigrateResult,
    MigrateResultProperties,
    MigrationEndpointMapping,
    MigrationErrorType,
    MigrationParameters,
    MigrationWebApplicationFirewallMapping,
    Operation,
    OperationDisplay,
    OperationProperties,
    Origin,
    OriginAuthenticationProperties,
    OriginGroup,
    OriginGroupOverride,
    OriginGroupOverrideAction,
    OriginGroupOverrideActionParameters,
    OriginGroupProperties,
    OriginGroupUpdateParameters,
    OriginGroupUpdatePropertiesParameters,
    OriginProperties,
    OriginUpdateParameters,
    OriginUpdatePropertiesParameters,
    PolicySettings,
    PostArgsMatchConditionParameters,
    Profile,
    ProfileChangeSkuWafMapping,
    ProfileLogScrubbing,
    ProfileProperties,
    ProfilePropertiesUpdateParameters,
    ProfileScrubbingRules,
    ProfileUpdateParameters,
    ProfileUpgradeParameters,
    ProxyResource,
    PurgeParameters,
    QueryStringMatchConditionParameters,
    RankingsResponse,
    RankingsResponseTablesItem,
    RankingsResponseTablesPropertiesItemsItem,
    RankingsResponseTablesPropertiesItemsMetricsItem,
    RateLimitRule,
    RateLimitRuleList,
    RemoteAddressMatchConditionParameters,
    RequestBodyMatchConditionParameters,
    RequestHeaderMatchConditionParameters,
    RequestMethodMatchConditionParameters,
    RequestSchemeMatchConditionParameters,
    RequestUriMatchConditionParameters,
    Resource,
    ResourceReference,
    ResourceUsage,
    ResourcesResponse,
    ResourcesResponseCustomDomainsItem,
    ResourcesResponseEndpointsItem,
    ResourcesResponseEndpointsPropertiesItemsItem,
    ResponseBasedOriginErrorDetectionParameters,
    Route,
    RouteConfigurationOverrideActionParameters,
    RouteProperties,
    RouteUpdateParameters,
    RouteUpdatePropertiesParameters,
    Rule,
    RuleProperties,
    RuleSet,
    RuleSetProperties,
    RuleUpdateParameters,
    RuleUpdatePropertiesParameters,
    Secret,
    SecretParameters,
    SecretProperties,
    SecurityPolicy,
    SecurityPolicyProperties,
    SecurityPolicyPropertiesParameters,
    SecurityPolicyUpdateParameters,
    SecurityPolicyUpdateProperties,
    SecurityPolicyWebApplicationFirewallAssociation,
    SecurityPolicyWebApplicationFirewallParameters,
    ServerPortMatchConditionParameters,
    ServiceSpecification,
    SharedPrivateLinkResourceProperties,
    Sku,
    SocketAddrMatchConditionParameters,
    SslProtocolMatchConditionParameters,
    SsoUri,
    SupportedOptimizationTypesListResult,
    SystemData,
    TrackedResource,
    UrlFileExtensionMatchConditionParameters,
    UrlFileNameMatchConditionParameters,
    UrlPathMatchConditionParameters,
    UrlRedirectAction,
    UrlRedirectActionParameters,
    UrlRewriteAction,
    UrlRewriteActionParameters,
    UrlSigningAction,
    UrlSigningActionParameters,
    UrlSigningKey,
    UrlSigningKeyParameters,
    UrlSigningParamIdentifier,
    Usage,
    UsageName,
    UserAssignedIdentity,
    UserManagedHttpsParameters,
    ValidateCustomDomainInput,
    ValidateCustomDomainOutput,
    ValidateProbeInput,
    ValidateProbeOutput,
    ValidateSecretInput,
    ValidateSecretOutput,
    WafMetricsResponse,
    WafMetricsResponseSeriesItem,
    WafMetricsResponseSeriesPropertiesItemsItem,
    WafRankingsResponse,
    WafRankingsResponseDataItem,
)

from ._enums import (  # type: ignore
    AFDEndpointProtocols,
    ActionType,
    AfdCertificateType,
    AfdCipherSuiteSetType,
    AfdCustomizedCipherSuiteForTls12,
    AfdCustomizedCipherSuiteForTls13,
    AfdMinimumTlsVersion,
    AfdProvisioningState,
    AfdQueryStringCachingBehavior,
    Algorithm,
    AutoGeneratedDomainNameLabelScope,
    CacheBehavior,
    CacheType,
    CanMigrateDefaultSku,
    CertificateSource,
    CertificateSourceParametersType,
    CertificateType,
    ClientPortOperator,
    CookiesOperator,
    CreatedByType,
    CustomDomainResourceState,
    CustomHttpsProvisioningState,
    CustomHttpsProvisioningSubstate,
    CustomRuleEnabledState,
    DeleteRule,
    DeliveryRuleActionEnum,
    DeliveryRuleActionParametersType,
    DeliveryRuleConditionParametersType,
    DeploymentStatus,
    DestinationProtocol,
    DomainValidationState,
    EnabledState,
    EndpointProvisioningState,
    EndpointResourceState,
    ForwardingProtocol,
    GeoFilterActions,
    HeaderAction,
    HealthProbeRequestType,
    HostNameOperator,
    HttpVersionOperator,
    HttpsRedirect,
    IsDeviceMatchConditionParametersMatchValuesItem,
    IsDeviceOperator,
    KeyVaultSigningKeyParametersTypeName,
    LinkToDefaultDomain,
    LogMetric,
    LogMetricsGranularity,
    LogMetricsGroupBy,
    LogRanking,
    LogRankingMetric,
    ManagedRuleEnabledState,
    ManagedServiceIdentityType,
    MatchProcessingBehavior,
    MatchVariable,
    MetricsGranularity,
    MetricsSeriesUnit,
    MinimumTlsVersion,
    Operator,
    OptimizationType,
    OriginAuthenticationType,
    OriginGroupProvisioningState,
    OriginGroupResourceState,
    OriginProvisioningState,
    OriginResourceState,
    ParamIndicator,
    PolicyEnabledState,
    PolicyMode,
    PolicyResourceState,
    PolicySettingsDefaultCustomBlockResponseStatusCode,
    PostArgsOperator,
    PrivateEndpointStatus,
    ProbeProtocol,
    ProfileProvisioningState,
    ProfileResourceState,
    ProfileScrubbingState,
    ProtocolType,
    ProvisioningState,
    QueryStringBehavior,
    QueryStringCachingBehavior,
    QueryStringOperator,
    RedirectType,
    RemoteAddressOperator,
    RequestBodyOperator,
    RequestHeaderOperator,
    RequestMethodMatchConditionParametersMatchValuesItem,
    RequestMethodOperator,
    RequestSchemeMatchConditionParametersMatchValuesItem,
    RequestSchemeMatchConditionParametersOperator,
    RequestUriOperator,
    ResourceType,
    ResourceUsageUnit,
    ResponseBasedDetectedErrorTypes,
    RuleCacheBehavior,
    RuleIsCompressionEnabled,
    RuleQueryStringCachingBehavior,
    ScrubbingRuleEntryMatchOperator,
    ScrubbingRuleEntryMatchVariable,
    ScrubbingRuleEntryState,
    SecretType,
    SecurityPolicyType,
    ServerPortOperator,
    SharedPrivateLinkResourceStatus,
    SkuName,
    SocketAddrOperator,
    SslProtocol,
    SslProtocolOperator,
    Status,
    Transform,
    TransformType,
    UpdateRule,
    UrlFileExtensionOperator,
    UrlFileNameOperator,
    UrlPathOperator,
    UsageUnit,
    WafAction,
    WafGranularity,
    WafMatchVariable,
    WafMetric,
    WafMetricsGranularity,
    WafMetricsSeriesUnit,
    WafRankingGroupBy,
    WafRankingType,
    WafRuleType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AFDDomain",
    "AFDDomainHttpsCustomizedCipherSuiteSet",
    "AFDDomainHttpsParameters",
    "AFDDomainProperties",
    "AFDDomainUpdateParameters",
    "AFDDomainUpdatePropertiesParameters",
    "AFDEndpoint",
    "AFDEndpointProperties",
    "AFDEndpointPropertiesUpdateParameters",
    "AFDEndpointUpdateParameters",
    "AFDOrigin",
    "AFDOriginGroup",
    "AFDOriginGroupProperties",
    "AFDOriginGroupUpdateParameters",
    "AFDOriginGroupUpdatePropertiesParameters",
    "AFDOriginProperties",
    "AFDOriginUpdateParameters",
    "AFDOriginUpdatePropertiesParameters",
    "AFDStateProperties",
    "ActivatedResourceReference",
    "AfdPurgeParameters",
    "AfdRouteCacheConfiguration",
    "AzureFirstPartyManagedCertificateParameters",
    "BatchRuleProperties",
    "CacheConfiguration",
    "CacheExpirationActionParameters",
    "CacheKeyQueryStringActionParameters",
    "CanMigrateParameters",
    "CanMigrateProperties",
    "CanMigrateResult",
    "CdnCertificateSourceParameters",
    "CdnEndpoint",
    "CdnManagedHttpsParameters",
    "CdnMigrationToAfdParameters",
    "CdnWebApplicationFirewallPolicy",
    "CdnWebApplicationFirewallPolicyPatchParameters",
    "CdnWebApplicationFirewallPolicyProperties",
    "CertificateSourceParameters",
    "CheckEndpointNameAvailabilityInput",
    "CheckEndpointNameAvailabilityOutput",
    "CheckHostNameAvailabilityInput",
    "CheckNameAvailabilityInput",
    "CheckNameAvailabilityOutput",
    "CidrIpAddress",
    "ClientPortMatchConditionParameters",
    "Components18OrqelSchemasWafmetricsresponsePropertiesSeriesItemsPropertiesDataItems",
    "Components1Gs0LlpSchemasMetricsresponsePropertiesSeriesItemsPropertiesDataItems",
    "ComponentsKpo1PjSchemasWafrankingsresponsePropertiesDataItemsPropertiesMetricsItems",
    "CompressionSettings",
    "ContinentsResponse",
    "ContinentsResponseContinentsItem",
    "ContinentsResponseCountryOrRegionsItem",
    "CookiesMatchConditionParameters",
    "CustomDomain",
    "CustomDomainHttpsParameters",
    "CustomDomainParameters",
    "CustomDomainProperties",
    "CustomDomainPropertiesParameters",
    "CustomRule",
    "CustomRuleList",
    "CustomerCertificateParameters",
    "DeepCreatedCustomDomain",
    "DeepCreatedCustomDomainProperties",
    "DeepCreatedOrigin",
    "DeepCreatedOriginGroup",
    "DeepCreatedOriginGroupProperties",
    "DeepCreatedOriginProperties",
    "DeliveryRule",
    "DeliveryRuleAction",
    "DeliveryRuleActionParameters",
    "DeliveryRuleCacheExpirationAction",
    "DeliveryRuleCacheKeyQueryStringAction",
    "DeliveryRuleClientPortCondition",
    "DeliveryRuleCondition",
    "DeliveryRuleConditionParameters",
    "DeliveryRuleCookiesCondition",
    "DeliveryRuleHostNameCondition",
    "DeliveryRuleHttpVersionCondition",
    "DeliveryRuleIsDeviceCondition",
    "DeliveryRulePostArgsCondition",
    "DeliveryRuleQueryStringCondition",
    "DeliveryRuleRemoteAddressCondition",
    "DeliveryRuleRequestBodyCondition",
    "DeliveryRuleRequestHeaderAction",
    "DeliveryRuleRequestHeaderCondition",
    "DeliveryRuleRequestMethodCondition",
    "DeliveryRuleRequestSchemeCondition",
    "DeliveryRuleRequestUriCondition",
    "DeliveryRuleResponseHeaderAction",
    "DeliveryRuleRouteConfigurationOverrideAction",
    "DeliveryRuleServerPortCondition",
    "DeliveryRuleSocketAddrCondition",
    "DeliveryRuleSslProtocolCondition",
    "DeliveryRuleUrlFileExtensionCondition",
    "DeliveryRuleUrlFileNameCondition",
    "DeliveryRuleUrlPathCondition",
    "DimensionProperties",
    "DomainValidationProperties",
    "EdgeNode",
    "EdgeNodeProperties",
    "Endpoint",
    "EndpointProperties",
    "EndpointPropertiesUpdateParameters",
    "EndpointPropertiesUpdateParametersDeliveryPolicy",
    "EndpointPropertiesUpdateParametersWebApplicationFirewallPolicyLink",
    "EndpointUpdateParameters",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "GeoFilter",
    "HeaderActionParameters",
    "HealthProbeParameters",
    "HostNameMatchConditionParameters",
    "HttpErrorRangeParameters",
    "HttpVersionMatchConditionParameters",
    "IpAddressGroup",
    "IsDeviceMatchConditionParameters",
    "KeyVaultCertificateSourceParameters",
    "KeyVaultSigningKeyParameters",
    "LoadBalancingSettingsParameters",
    "LoadParameters",
    "LogSpecification",
    "ManagedCertificateParameters",
    "ManagedRuleDefinition",
    "ManagedRuleGroupDefinition",
    "ManagedRuleGroupOverride",
    "ManagedRuleOverride",
    "ManagedRuleSet",
    "ManagedRuleSetDefinition",
    "ManagedRuleSetDefinitionProperties",
    "ManagedRuleSetList",
    "ManagedServiceIdentity",
    "MatchCondition",
    "MetricAvailability",
    "MetricSpecification",
    "MetricsResponse",
    "MetricsResponseSeriesItem",
    "MetricsResponseSeriesPropertiesItemsItem",
    "MigrateResult",
    "MigrateResultProperties",
    "MigrationEndpointMapping",
    "MigrationErrorType",
    "MigrationParameters",
    "MigrationWebApplicationFirewallMapping",
    "Operation",
    "OperationDisplay",
    "OperationProperties",
    "Origin",
    "OriginAuthenticationProperties",
    "OriginGroup",
    "OriginGroupOverride",
    "OriginGroupOverrideAction",
    "OriginGroupOverrideActionParameters",
    "OriginGroupProperties",
    "OriginGroupUpdateParameters",
    "OriginGroupUpdatePropertiesParameters",
    "OriginProperties",
    "OriginUpdateParameters",
    "OriginUpdatePropertiesParameters",
    "PolicySettings",
    "PostArgsMatchConditionParameters",
    "Profile",
    "ProfileChangeSkuWafMapping",
    "ProfileLogScrubbing",
    "ProfileProperties",
    "ProfilePropertiesUpdateParameters",
    "ProfileScrubbingRules",
    "ProfileUpdateParameters",
    "ProfileUpgradeParameters",
    "ProxyResource",
    "PurgeParameters",
    "QueryStringMatchConditionParameters",
    "RankingsResponse",
    "RankingsResponseTablesItem",
    "RankingsResponseTablesPropertiesItemsItem",
    "RankingsResponseTablesPropertiesItemsMetricsItem",
    "RateLimitRule",
    "RateLimitRuleList",
    "RemoteAddressMatchConditionParameters",
    "RequestBodyMatchConditionParameters",
    "RequestHeaderMatchConditionParameters",
    "RequestMethodMatchConditionParameters",
    "RequestSchemeMatchConditionParameters",
    "RequestUriMatchConditionParameters",
    "Resource",
    "ResourceReference",
    "ResourceUsage",
    "ResourcesResponse",
    "ResourcesResponseCustomDomainsItem",
    "ResourcesResponseEndpointsItem",
    "ResourcesResponseEndpointsPropertiesItemsItem",
    "ResponseBasedOriginErrorDetectionParameters",
    "Route",
    "RouteConfigurationOverrideActionParameters",
    "RouteProperties",
    "RouteUpdateParameters",
    "RouteUpdatePropertiesParameters",
    "Rule",
    "RuleProperties",
    "RuleSet",
    "RuleSetProperties",
    "RuleUpdateParameters",
    "RuleUpdatePropertiesParameters",
    "Secret",
    "SecretParameters",
    "SecretProperties",
    "SecurityPolicy",
    "SecurityPolicyProperties",
    "SecurityPolicyPropertiesParameters",
    "SecurityPolicyUpdateParameters",
    "SecurityPolicyUpdateProperties",
    "SecurityPolicyWebApplicationFirewallAssociation",
    "SecurityPolicyWebApplicationFirewallParameters",
    "ServerPortMatchConditionParameters",
    "ServiceSpecification",
    "SharedPrivateLinkResourceProperties",
    "Sku",
    "SocketAddrMatchConditionParameters",
    "SslProtocolMatchConditionParameters",
    "SsoUri",
    "SupportedOptimizationTypesListResult",
    "SystemData",
    "TrackedResource",
    "UrlFileExtensionMatchConditionParameters",
    "UrlFileNameMatchConditionParameters",
    "UrlPathMatchConditionParameters",
    "UrlRedirectAction",
    "UrlRedirectActionParameters",
    "UrlRewriteAction",
    "UrlRewriteActionParameters",
    "UrlSigningAction",
    "UrlSigningActionParameters",
    "UrlSigningKey",
    "UrlSigningKeyParameters",
    "UrlSigningParamIdentifier",
    "Usage",
    "UsageName",
    "UserAssignedIdentity",
    "UserManagedHttpsParameters",
    "ValidateCustomDomainInput",
    "ValidateCustomDomainOutput",
    "ValidateProbeInput",
    "ValidateProbeOutput",
    "ValidateSecretInput",
    "ValidateSecretOutput",
    "WafMetricsResponse",
    "WafMetricsResponseSeriesItem",
    "WafMetricsResponseSeriesPropertiesItemsItem",
    "WafRankingsResponse",
    "WafRankingsResponseDataItem",
    "AFDEndpointProtocols",
    "ActionType",
    "AfdCertificateType",
    "AfdCipherSuiteSetType",
    "AfdCustomizedCipherSuiteForTls12",
    "AfdCustomizedCipherSuiteForTls13",
    "AfdMinimumTlsVersion",
    "AfdProvisioningState",
    "AfdQueryStringCachingBehavior",
    "Algorithm",
    "AutoGeneratedDomainNameLabelScope",
    "CacheBehavior",
    "CacheType",
    "CanMigrateDefaultSku",
    "CertificateSource",
    "CertificateSourceParametersType",
    "CertificateType",
    "ClientPortOperator",
    "CookiesOperator",
    "CreatedByType",
    "CustomDomainResourceState",
    "CustomHttpsProvisioningState",
    "CustomHttpsProvisioningSubstate",
    "CustomRuleEnabledState",
    "DeleteRule",
    "DeliveryRuleActionEnum",
    "DeliveryRuleActionParametersType",
    "DeliveryRuleConditionParametersType",
    "DeploymentStatus",
    "DestinationProtocol",
    "DomainValidationState",
    "EnabledState",
    "EndpointProvisioningState",
    "EndpointResourceState",
    "ForwardingProtocol",
    "GeoFilterActions",
    "HeaderAction",
    "HealthProbeRequestType",
    "HostNameOperator",
    "HttpVersionOperator",
    "HttpsRedirect",
    "IsDeviceMatchConditionParametersMatchValuesItem",
    "IsDeviceOperator",
    "KeyVaultSigningKeyParametersTypeName",
    "LinkToDefaultDomain",
    "LogMetric",
    "LogMetricsGranularity",
    "LogMetricsGroupBy",
    "LogRanking",
    "LogRankingMetric",
    "ManagedRuleEnabledState",
    "ManagedServiceIdentityType",
    "MatchProcessingBehavior",
    "MatchVariable",
    "MetricsGranularity",
    "MetricsSeriesUnit",
    "MinimumTlsVersion",
    "Operator",
    "OptimizationType",
    "OriginAuthenticationType",
    "OriginGroupProvisioningState",
    "OriginGroupResourceState",
    "OriginProvisioningState",
    "OriginResourceState",
    "ParamIndicator",
    "PolicyEnabledState",
    "PolicyMode",
    "PolicyResourceState",
    "PolicySettingsDefaultCustomBlockResponseStatusCode",
    "PostArgsOperator",
    "PrivateEndpointStatus",
    "ProbeProtocol",
    "ProfileProvisioningState",
    "ProfileResourceState",
    "ProfileScrubbingState",
    "ProtocolType",
    "ProvisioningState",
    "QueryStringBehavior",
    "QueryStringCachingBehavior",
    "QueryStringOperator",
    "RedirectType",
    "RemoteAddressOperator",
    "RequestBodyOperator",
    "RequestHeaderOperator",
    "RequestMethodMatchConditionParametersMatchValuesItem",
    "RequestMethodOperator",
    "RequestSchemeMatchConditionParametersMatchValuesItem",
    "RequestSchemeMatchConditionParametersOperator",
    "RequestUriOperator",
    "ResourceType",
    "ResourceUsageUnit",
    "ResponseBasedDetectedErrorTypes",
    "RuleCacheBehavior",
    "RuleIsCompressionEnabled",
    "RuleQueryStringCachingBehavior",
    "ScrubbingRuleEntryMatchOperator",
    "ScrubbingRuleEntryMatchVariable",
    "ScrubbingRuleEntryState",
    "SecretType",
    "SecurityPolicyType",
    "ServerPortOperator",
    "SharedPrivateLinkResourceStatus",
    "SkuName",
    "SocketAddrOperator",
    "SslProtocol",
    "SslProtocolOperator",
    "Status",
    "Transform",
    "TransformType",
    "UpdateRule",
    "UrlFileExtensionOperator",
    "UrlFileNameOperator",
    "UrlPathOperator",
    "UsageUnit",
    "WafAction",
    "WafGranularity",
    "WafMatchVariable",
    "WafMetric",
    "WafMetricsGranularity",
    "WafMetricsSeriesUnit",
    "WafRankingGroupBy",
    "WafRankingType",
    "WafRuleType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
