# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    ApiEntityReference,
    ApplicationGateway,
    ApplicationGatewayBackendAddressPool,
    ArmResource,
    AzureFileVolume,
    CachedImages,
    Capabilities,
    CapabilitiesCapabilities,
    CloudError,
    CloudErrorBody,
    ConfidentialComputeProperties,
    ConfigMap,
    Container,
    ContainerAttachResponse,
    ContainerExec,
    ContainerExecRequest,
    ContainerExecRequestTerminalSize,
    ContainerExecResponse,
    ContainerGroup,
    ContainerGroupDiagnostics,
    ContainerGroupIdentity,
    ContainerGroupProfile,
    ContainerGroupProfilePatch,
    ContainerGroupProfileProperties,
    ContainerGroupProfileReferenceDefinition,
    ContainerGroupProfileStub,
    ContainerGroupProperties,
    ContainerGroupPropertiesInstanceView,
    ContainerGroupSubnetId,
    ContainerHttpGet,
    ContainerPort,
    ContainerProbe,
    ContainerProperties,
    ContainerPropertiesInstanceView,
    ContainerState,
    DeploymentExtensionSpec,
    DeploymentExtensionSpecProperties,
    DnsConfiguration,
    ElasticProfile,
    ElasticProfileContainerGroupNamingPolicy,
    ElasticProfileContainerGroupNamingPolicyGuidNamingPolicy,
    EncryptionProperties,
    EnvironmentVariable,
    ErrorAdditionalInfo,
    ErrorDetail,
    ErrorResponse,
    Event,
    FileShare,
    FileShareProperties,
    GitRepoVolume,
    GpuResource,
    HttpHeader,
    IdentityAccessControl,
    IdentityAcls,
    ImageRegistryCredential,
    InitContainerDefinition,
    InitContainerPropertiesDefinition,
    InitContainerPropertiesDefinitionInstanceView,
    IpAddress,
    LoadBalancer,
    LoadBalancerBackendAddressPool,
    LogAnalytics,
    Logs,
    ManagedServiceIdentity,
    NGroup,
    NGroupCGPropertyContainer,
    NGroupCGPropertyContainerProperties,
    NGroupCGPropertyVolume,
    NGroupContainerGroupProperties,
    NGroupIdentity,
    NGroupPatch,
    NGroupProperties,
    NetworkProfile,
    Operation,
    OperationDisplay,
    PlacementProfile,
    Port,
    ProxyResource,
    Resource,
    ResourceLimits,
    ResourceRequests,
    ResourceRequirements,
    SandboxGroup,
    SandboxGroupAccessToken,
    SandboxGroupNetworkProfile,
    SandboxGroupProperties,
    SandboxGroupTagsUpdate,
    SecretReference,
    SecurityContextCapabilitiesDefinition,
    SecurityContextDefinition,
    StandbyPoolProfileDefinition,
    StorageProfile,
    SubnetReference,
    SystemData,
    TrackedResource,
    UpdateProfile,
    UpdateProfileRollingUpdateProfile,
    Usage,
    UsageName,
    UserAssignedIdentities,
    UserAssignedIdentity,
    Volume,
    VolumeMount,
)

from ._enums import (  # type: ignore
    AzureFileShareAccessTier,
    AzureFileShareAccessType,
    ContainerGroupIpAddressType,
    ContainerGroupNetworkProtocol,
    ContainerGroupPriority,
    ContainerGroupRestartPolicy,
    ContainerGroupSku,
    ContainerInstanceOperationsOrigin,
    ContainerNetworkProtocol,
    CreatedByType,
    DnsNameLabelReusePolicy,
    GpuSku,
    IdentityAccessLevel,
    LogAnalyticsLogType,
    ManagedServiceIdentityType,
    NGroupProvisioningState,
    NGroupUpdateMode,
    OperatingSystemTypes,
    ResourceIdentityType,
    SandboxGroupProvisioningState,
    Scheme,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ApiEntityReference",
    "ApplicationGateway",
    "ApplicationGatewayBackendAddressPool",
    "ArmResource",
    "AzureFileVolume",
    "CachedImages",
    "Capabilities",
    "CapabilitiesCapabilities",
    "CloudError",
    "CloudErrorBody",
    "ConfidentialComputeProperties",
    "ConfigMap",
    "Container",
    "ContainerAttachResponse",
    "ContainerExec",
    "ContainerExecRequest",
    "ContainerExecRequestTerminalSize",
    "ContainerExecResponse",
    "ContainerGroup",
    "ContainerGroupDiagnostics",
    "ContainerGroupIdentity",
    "ContainerGroupProfile",
    "ContainerGroupProfilePatch",
    "ContainerGroupProfileProperties",
    "ContainerGroupProfileReferenceDefinition",
    "ContainerGroupProfileStub",
    "ContainerGroupProperties",
    "ContainerGroupPropertiesInstanceView",
    "ContainerGroupSubnetId",
    "ContainerHttpGet",
    "ContainerPort",
    "ContainerProbe",
    "ContainerProperties",
    "ContainerPropertiesInstanceView",
    "ContainerState",
    "DeploymentExtensionSpec",
    "DeploymentExtensionSpecProperties",
    "DnsConfiguration",
    "ElasticProfile",
    "ElasticProfileContainerGroupNamingPolicy",
    "ElasticProfileContainerGroupNamingPolicyGuidNamingPolicy",
    "EncryptionProperties",
    "EnvironmentVariable",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "Event",
    "FileShare",
    "FileShareProperties",
    "GitRepoVolume",
    "GpuResource",
    "HttpHeader",
    "IdentityAccessControl",
    "IdentityAcls",
    "ImageRegistryCredential",
    "InitContainerDefinition",
    "InitContainerPropertiesDefinition",
    "InitContainerPropertiesDefinitionInstanceView",
    "IpAddress",
    "LoadBalancer",
    "LoadBalancerBackendAddressPool",
    "LogAnalytics",
    "Logs",
    "ManagedServiceIdentity",
    "NGroup",
    "NGroupCGPropertyContainer",
    "NGroupCGPropertyContainerProperties",
    "NGroupCGPropertyVolume",
    "NGroupContainerGroupProperties",
    "NGroupIdentity",
    "NGroupPatch",
    "NGroupProperties",
    "NetworkProfile",
    "Operation",
    "OperationDisplay",
    "PlacementProfile",
    "Port",
    "ProxyResource",
    "Resource",
    "ResourceLimits",
    "ResourceRequests",
    "ResourceRequirements",
    "SandboxGroup",
    "SandboxGroupAccessToken",
    "SandboxGroupNetworkProfile",
    "SandboxGroupProperties",
    "SandboxGroupTagsUpdate",
    "SecretReference",
    "SecurityContextCapabilitiesDefinition",
    "SecurityContextDefinition",
    "StandbyPoolProfileDefinition",
    "StorageProfile",
    "SubnetReference",
    "SystemData",
    "TrackedResource",
    "UpdateProfile",
    "UpdateProfileRollingUpdateProfile",
    "Usage",
    "UsageName",
    "UserAssignedIdentities",
    "UserAssignedIdentity",
    "Volume",
    "VolumeMount",
    "AzureFileShareAccessTier",
    "AzureFileShareAccessType",
    "ContainerGroupIpAddressType",
    "ContainerGroupNetworkProtocol",
    "ContainerGroupPriority",
    "ContainerGroupRestartPolicy",
    "ContainerGroupSku",
    "ContainerInstanceOperationsOrigin",
    "ContainerNetworkProtocol",
    "CreatedByType",
    "DnsNameLabelReusePolicy",
    "GpuSku",
    "IdentityAccessLevel",
    "LogAnalyticsLogType",
    "ManagedServiceIdentityType",
    "NGroupProvisioningState",
    "NGroupUpdateMode",
    "OperatingSystemTypes",
    "ResourceIdentityType",
    "SandboxGroupProvisioningState",
    "Scheme",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
